package com.agentfactory.afapl.compiler;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import com.agentfactory.afapl.interpreter.AFAPLAchieveStep;
import com.agentfactory.afapl.interpreter.AFAPLBasicStep;
import com.agentfactory.afapl.interpreter.AFAPLUpdateStep;
import com.agentfactory.afapl.interpreter.Utilities;
import com.agentfactory.clf.lang.AND;
import com.agentfactory.clf.lang.IFormula;
import com.agentfactory.clf.lang.Predicate;
import com.agentfactory.clf.lang.Variable;
import com.agentfactory.clf.plans.AssignmentStep;
import com.agentfactory.clf.plans.BlockStep;
import com.agentfactory.clf.plans.DoStep;
import com.agentfactory.clf.plans.DurativeStep;
import com.agentfactory.clf.plans.ForEachStep;
import com.agentfactory.clf.plans.IPlanStep;
import com.agentfactory.clf.plans.IfStep;
import com.agentfactory.clf.plans.MultiAssignmentStep;
import com.agentfactory.clf.plans.NewListStep;
import com.agentfactory.clf.plans.QueryStep;
import com.agentfactory.clf.plans.TryRecoverStep;
import com.agentfactory.clf.plans.WaitStep;
import com.agentfactory.clf.plans.WhenStep;
import com.agentfactory.clf.plans.WhileStep;

public class PlanBuilderVisitor implements AFAPLParserVisitor {

	@SuppressWarnings("unchecked")
	@Override
	public Object visit(SimpleNode node, Object data) {
		if (node.toString().equals("Plan")) {
			List<IPlanStep> plan = new LinkedList<IPlanStep>();
			for (int i=0; i<node.jjtGetNumChildren(); i++) {
				plan.add((IPlanStep) visit((SimpleNode) node.jjtGetChild(i), null));
			}
			return plan;
		} else if (node.toString().equals("Sequence")) {
			List<IPlanStep> plan = new LinkedList<IPlanStep>();
			for (int i=0; i<node.jjtGetNumChildren(); i++) {
				Object obj = visit((SimpleNode) node.jjtGetChild(i), null);
				if (IPlanStep.class.isInstance(obj)) {
					plan.add((IPlanStep) obj);
				} else {
					plan.addAll((List<IPlanStep>) obj);
				}
			}
			return plan;
		} else if (node.toString().equals("Block")) {
			return new BlockStep((List<IPlanStep>) visit((SimpleNode) node.jjtGetChild(0), null));
		} else if (node.toString().equals("Query")) {
			return new QueryStep((IFormula) new SentenceBuilderVisitor().visit((SimpleNode) node.jjtGetChild(0), null));
		} else if (node.toString().equals("SingleStatement")) {
			return visit((SimpleNode) node.jjtGetChild(0), data);
		} else if (node.toString().equals("ComplexStatement")) {
			return visit((SimpleNode) node.jjtGetChild(0), data);
		} else if (node.toString().equals("BasicStatement")) {
			return visit((SimpleNode) node.jjtGetChild(0), data);
		} else if (node.toString().equals("AchieveGoal")) {
			String fos = "!";
			fos += (String) new PredicateBuilderVisitor().visit((SimpleNode) node.jjtGetChild(0), null);
			return new AFAPLAchieveStep(Utilities.factory.createFormula(fos));
		} else if (node.toString().equals("BeliefUpdate")) {
				String op = (String) node.jjtGetValue();
				SimpleNode _node = (SimpleNode) node.jjtGetChild(0);
				String fos = "";
				if (_node.toString().equals("Predicate")) {
					fos = (String) new PredicateBuilderVisitor().visit(_node, null);
				} else if (_node.toString().equals("Goal")) {
					fos = (String) _node.jjtGetValue();
					fos += (String) new PredicateBuilderVisitor().visit((SimpleNode) _node.jjtGetChild(0), null);
				}
				return new AFAPLUpdateStep(op, Utilities.factory.createFormula(fos));
		} else if (node.toString().equals("Predicate")) {
			String fos = (String) new PredicateBuilderVisitor().visit(node, null);
			return new AFAPLBasicStep(Utilities.factory.createFormula(fos));
		} else if (node.toString().equals("When")) {
			return new WhenStep(
					(AND) new SentenceBuilderVisitor().visit((SimpleNode) node.jjtGetChild(0), null),
					(IPlanStep) visit((SimpleNode) node.jjtGetChild(1), null));
		} else if (node.toString().equals("While")) {
			return new WhileStep(
					(AND) new SentenceBuilderVisitor().visit((SimpleNode) node.jjtGetChild(0), null),
					(IPlanStep) visit((SimpleNode) node.jjtGetChild(1), null));
		} else if (node.toString().equals("TryRecover")) {
			return new TryRecoverStep(
					(IPlanStep) visit((SimpleNode) node.jjtGetChild(0), null),
					(IPlanStep) visit((SimpleNode) node.jjtGetChild(1), null));
		} else if (node.toString().equals("If")) {
			AND and = (AND) new SentenceBuilderVisitor().visit((SimpleNode) node.jjtGetChild(0), null);
			IPlanStep ifStep = (IPlanStep) visit((SimpleNode) node.jjtGetChild(1), null);
			IPlanStep elseStep = null;
			if (node.jjtGetNumChildren() == 3) {
				elseStep = (IPlanStep) visit((SimpleNode) node.jjtGetChild(2), null);
			}
			return new IfStep(and, ifStep, elseStep);
		} else if (node.toString().equals("ForEach")) {
			return new ForEachStep(
					(AND) new SentenceBuilderVisitor().visit((SimpleNode) node.jjtGetChild(0), null),
					(IPlanStep) visit((SimpleNode) node.jjtGetChild(1), null));
		} else if (node.toString().equals("Wait")) {
			return new WaitStep((AND) new SentenceBuilderVisitor().visit((SimpleNode) node.jjtGetChild(0), null));
		} else if (node.toString().equals("Durative")) {
			if (node.jjtGetChild(0).toString().equals("Predicate")) {
				return new DurativeStep(Utilities.factory.createFormula((String) new PredicateBuilderVisitor().visit((SimpleNode) node.jjtGetChild(0), null)));
			} else {
				System.out.println("WARNING: Unsupported Operation: ");
				node.dump("");
			}
		} else if (node.toString().equals("Assignment")) {
			String variable = (String) node.jjtGetValue();
			if (node.jjtGetChild(0).toString().equals("Term")) {
				String term = (String) new PredicateBuilderVisitor().visit((SimpleNode) node.jjtGetChild(0), null);
				return new AssignmentStep((Variable) Utilities.factory.createTerm(variable), Utilities.factory.createTerm(term), false);
			} else if (node.jjtGetChild(0).toString().equals("NewList")) {
				SimpleNode list = (SimpleNode) node.jjtGetChild(0);
				String term = (String) new PredicateBuilderVisitor().visit((SimpleNode) list.jjtGetChild(0), null);
				return new NewListStep(Utilities.factory.createTerm(term),
						(AND) new SentenceBuilderVisitor().visit((SimpleNode) list.jjtGetChild(1), null),
						Utilities.factory.createTerm(variable));
			} else if (node.jjtGetChild(0).toString().equals("Durative")) {
				SimpleNode durative = (SimpleNode) node.jjtGetChild(0);
				String term = (String) new PredicateBuilderVisitor().visit((SimpleNode) durative.jjtGetChild(0), null);
				return new AssignmentStep((Variable) Utilities.factory.createTerm(variable), Utilities.factory.createTerm(term), true);
			} else {
				System.out.println("WARNING: Unsupported Operation: ");
				node.dump("");
			}
		} else if (node.toString().equals("MultiAssignment")) {
			List<Variable> variables = new ArrayList<Variable>();
			new VariableListVisitor().visit((SimpleNode) node.jjtGetChild(0), variables);
			if (node.jjtGetChild(1).toString().equals("Term")) {
				return new MultiAssignmentStep(variables, Utilities.factory.createTerm((String) new PredicateBuilderVisitor().visit((SimpleNode) node.jjtGetChild(1), null)), false);
			} else if (node.jjtGetChild(1).toString().equals("Durative")) {
				SimpleNode durative = (SimpleNode) node.jjtGetChild(1);
				String term = (String) new PredicateBuilderVisitor().visit((SimpleNode) durative.jjtGetChild(0), null);
				return new MultiAssignmentStep(variables, Utilities.factory.createTerm(term), true);
			}
		} else if (node.toString().equals("Do")) {
			String module = (String) new PredicateBuilderVisitor().visit((SimpleNode) node.jjtGetChild(0), null);
			String action = (String) new PredicateBuilderVisitor().visit((SimpleNode) node.jjtGetChild(1), null);
			return new DoStep(Utilities.factory.createTerm(module), (Predicate) Utilities.factory.createFormula(action));
		}
		return null;
	}
}
