package com.agentfactory.afapl.interpreter;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.agentfactory.afapl.debugger.AFAPLInspectorFactory;
import com.agentfactory.afapl.debugger.AFAPLStateManagerFactory;
import com.agentfactory.platform.core.ConfigurationException;
import com.agentfactory.visualiser.DebuggerRunConfiguration;

public class AFAPLDebugConfiguration extends DebuggerRunConfiguration {
	private String name;
	Map<String, String> designs;
	Map<String, List<String>> beliefs;
	
	public AFAPLDebugConfiguration(String name, Map<String, String> designs) {
		this(name, designs, new HashMap<String, List<String>>());
	}
	
	public AFAPLDebugConfiguration(String name, Map<String, String> designs, Map<String, List<String>> beliefs) {
		this.name = name;
		this.designs = designs;
		this.beliefs = beliefs;
	}
	
	public String getName() {
		return name;
	}
	
	public String getDomain() {
		return "agentspeak.ucd.ie";
	}
	
	public void configure() throws ConfigurationException {
		addLanguageSupport(new AFAPLInspectorFactory(), new AFAPLStateManagerFactory());
		
		super.configure();
		
		addArchitectureFactory(new AFAPLArchitectureFactory());
		
        for (String design: designs.keySet()) {
        	addAgent(design, designs.get(design));
        }
        
        for (String key : beliefs.keySet()) {
        	List<String> list = beliefs.get(key);
        	for (String belief: list) {
        		initAgent(key, belief);
        	}
        }
	}
}
