package com.agentfactory.afapl.interpreter;

import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import com.agentfactory.clf.lang.IFormula;
import com.agentfactory.clf.lang.Variable;
import com.agentfactory.clf.reasoner.Bindings;

public class Goal implements IFormula {
	public static final int ACHIEVE									= 0;
	public static final int MAINTAIN								= 1;
	
	private int type;
	private IFormula formula;

	public Goal(int type, IFormula formula) {
		this.type = type;
		this.formula = formula;
	}

	@Override
	public IFormula apply(Bindings set) {
		return new Goal(type, formula.apply(set));
	}

	@Override
	public List<IFormula> asList() {
		List<IFormula> list = new LinkedList<IFormula>();
		list.add(this);
		return list;
	}
	
	public IFormula formula() {
		return formula;
	}

	@Override
	public Set<Variable> getVariables() {
		return formula.getVariables();
	}

	@Override
	public boolean isFalse() {
		return formula.isFalse();
	}

	@Override
	public boolean isTrue() {
		return formula.isTrue();
	}

	@Override
	public boolean matches(IFormula formula) {
		if (Goal.class.isInstance(formula)) {
			return (type == ((Goal) formula).type) && this.formula.matches(((Goal) formula).formula); 
		}
		return false;
	}

	@Override
	public String type() {
		return formula.type();
	}
	
	public int goalType() {
		return type;
	}
}
