/*
 * ClassCheckVisitor.java
 *
 * Created on 19 June 2006, 11:46
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.agentfactory.afapl2.compiler;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.afapl2.compiler.parser.Visitor;
import com.agentfactory.afapl2.compiler.parser.node.ImportNode;
import com.agentfactory.afapl2.compiler.parser.node.Node;
import com.agentfactory.afapl2.compiler.parser.node.ProgramNode;
/**
 * This first phase is concerned with the expansion of the parse tree to include
 * any imported files.
 *
 * @author Rem Collier
 */
public class Phase1Visitor extends Visitor {
    private List<String> imports;
    
    public Phase1Visitor(List<AFAPL2SyntaxError> errors, List<AFAPL2SyntaxError> warnings) {
        super(errors,warnings);
        imports = new LinkedList<String>();
    }

    @Override
	public void visit(Node node) {
        if (node instanceof ProgramNode) {
            visit((ProgramNode) node);
        } else if (node instanceof ImportNode) {
            visit((ImportNode) node);
        } else {
            throw new RuntimeException("Attempted to visit a Node instance: " + node);
        }
    }
    
    private void visit(ProgramNode node) {
        for (ImportNode n: node.getImports()) {
            n.accept(this);
            
        }
    }
    
    private void visit(ImportNode node) {
        String file = "/" + node.getAFAPL2File().replace('.', '/') + ".afapl2";
         if (this.getClass().getResource(file) == null) {
            addError(node.getSource(), node.getLine(),"IMPORT references unknown AFAPL2 file: " + node.getAFAPL2File());
         } else {
            if (!imports.contains(node.getAFAPL2File())) {
                imports.add(node.getAFAPL2File());
            }
         }
    }
    
    public List<String> getImports() {
        return imports;
    }
}
