/**
 * Title:       LexicalAnalyserException.java
 * Copyright:   Copyright (c) 1996-2004 The Agent Factory Working Group. All rights reserved.
 * Licence:     This file is free software; you can redistribute it and/or modify
 *              it under the terms of the GNU Lesser General Public License as published by
 *              the Free Software Foundation; either version 2.1, or (at your option)
 *              any later version.
 *
 *              This file is distributed in the hope that it will be useful,
 *              but WITHOUT ANY WARRANTY; without even the implied warranty of
 *              MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *              GNU Lesser General Public License for more details.
 *
 *              You should have received a copy of the GNU Lesser General Public License
 *              along with Agent Factory; see the file COPYING.  If not, write to
 *              the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *              Boston, MA 02111-1307, USA.
 */

package com.agentfactory.afapl2.compiler.parser;

import com.agentfactory.afapl2.compiler.parser.token.Token;
/**
 *
 * @author  Rem Collier
 */
public class ParseTreeException extends Exception {
    private String type;
    private String message;
    private Token token;
    
    /** Creates a new instance of LexicalAnalyserException */
    public ParseTreeException(String message, String type, Token token) {
        this.message = message;
        this.type = type;
        this.token = token;
    }
    
    public ParseTreeException(String message) {
        super(message);
    }
    
    public String getSource() {
    	return token.getReader().toString();
    }
    
    public long getLineNumber() {
    	return token.getLine();
    }
    
    public String getErrorMessage() {
    	return message;
    }
    
    public String getIssue() {
    	return "symbol: " + type + " " + token.getText();
    }
    
    public void printError() {
        System.out.println(token.getReader() + ":" + token.getLine() + ": " + message);
        System.out.println("symbol: " + type + " " + token.getText());
    }
    
    @Override
	public String toString() {
        return token.getReader() + ":" + token.getLine() + ": " + message +
                "\nsymbol: " + type + " " + token.getText();
    }
    
}
