/*
 * ParseTreeGenerator.java
 *
 * Created on 13 June 2006, 14:30
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.agentfactory.afapl2.compiler.parser;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.afapl2.compiler.AFAPL2SyntaxError;
import com.agentfactory.afapl2.compiler.parser.node.AndNode;
import com.agentfactory.afapl2.compiler.parser.node.Node;
import com.agentfactory.afapl2.compiler.parser.node.ProgramNode;
import com.agentfactory.afapl2.compiler.parser.token.Token;

/**
 *
 * @author Administrator
 */
public class ParseTreeGenerator {
    private ProgramNode root;
    private List<AFAPL2SyntaxError> errors;
    
    /** Creates a new instance of ParseTreeGenerator */
    public ParseTreeGenerator() {
        errors = new LinkedList<AFAPL2SyntaxError>();
    }
    
    public boolean generateParseTree(LexicalAnalyser analyser) {
        Node node = null;
        boolean success = false;
        
        // Create a new root node...
        root = new ProgramNode(analyser.getSource());
        
        try {
            Token token = analyser.getNextToken();
            while (token != null) {
                node = Node.create(token, analyser);
                node = ParserHelper.checkForLogicOperators(";", node, token, analyser);
                ParserHelper.punctuationCheck(";", "Missing ; at end of statement: " + node, "expected ; but got", analyser);
                if (node instanceof AndNode) {
                    throw new ParseTreeException("Invalid Construct", "expected => but got ", token);
                }
                root.addChild(node);
                token = analyser.getNextToken();
            }
            success = true;
        } catch (LexicalAnalyserException lae) {
            errors.add(new AFAPL2SyntaxError(AFAPL2SyntaxError.ERROR, lae.getSource(), lae.getLineNumber(), "SyntaxError", lae.getMessage()));
        } catch (ParseTreeException pte) {
            errors.add(new AFAPL2SyntaxError(AFAPL2SyntaxError.ERROR, pte.getSource(), pte.getLineNumber(), pte.getErrorMessage(), pte.getIssue()));
        }
        
        return success;
    }
    
    public ProgramNode getRoot() {
        return root;
    }
    
    public List<AFAPL2SyntaxError> getErrors() {
        return errors;
    }
}
