/*
 * ParserHelper.java
 *
 * Created on 13 June 2006, 20:29
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.agentfactory.afapl2.compiler.parser;

import com.agentfactory.afapl2.compiler.parser.node.AndNode;
import com.agentfactory.afapl2.compiler.parser.node.ImpliesNode;
import com.agentfactory.afapl2.compiler.parser.node.Node;
import com.agentfactory.afapl2.compiler.parser.token.PunctuationToken;
import com.agentfactory.afapl2.compiler.parser.token.Token;

/**
 *
 * @author Administrator
 */
public class ParserHelper {
    public static Node checkForLogicOperators(String terminator, Node node, Token token, LexicalAnalyser analyser) throws LexicalAnalyserException, ParseTreeException {
        AndNode andNode = null;
        ImpliesNode impliesNode = null;
        
        // Check if we have reached the end of a construct, or if we are in a larger
        // construct (i.e. a rule of some sort)
        token = analyser.getNextToken();
        while ((token != null) && (!token.getText().equals(terminator))) {
            if (token.getText().equals("&")) {
                if (andNode == null) {
                    andNode = new AndNode();
                    andNode.addNode(node);
                }
                token = analyser.getNextToken();
                andNode.addNode(Node.create(token, analyser));
            } else if (token.getText().equals("=>")) {
                if (terminator.equals(",")) {
                    throw new ParseTreeException("Cannot use an implication as a term", "expected , or & but got", token);
                }
                
                if (andNode == null) {
                    andNode = new AndNode();
                    andNode.addNode(node);
                }
                
                if (impliesNode == null) {
                    impliesNode = new ImpliesNode();
                    impliesNode.setLHS(andNode);
                    
                    token = analyser.getNextToken();
                    impliesNode.setRHS(Node.create(token, analyser));
                } else {
                    throw new ParseTreeException("Illegal use of =>", "did not expect", token);
                }
            } else {
                throw new ParseTreeException("Invalid Construct", "expected &, =>, or " + terminator + " but got ", token);
            }
            
            token = analyser.getNextToken();
        }
        
        if (token == null) {
            throw new LexicalAnalyserException(analyser.getClass().toString(), 0, "Semi-colon expected before the end of file.");
        }
        
        if (terminator.equals(",") || terminator.equals(")")) {
            if (andNode == null) {
                andNode = new AndNode();
                andNode.addNode(node);
            }
            node = andNode;
        } else if (terminator.equals(";")) {
            if (andNode != null) {
                // Check that implies node is not null as otherwise we have a
                // badly formed construct.
                if (impliesNode != null) {
                    node = impliesNode;
                } else {
                    node = andNode;
//                    throw new ParseTreeException("Invalid Construct", "expected => but got ", token);
                }
            }
        }
        
        analyser.back(token);
        
        return node;
    }
    
    public static void punctuationCheck(String text, String error, String message, LexicalAnalyser analyser) throws LexicalAnalyserException, ParseTreeException {
        Token token = analyser.getNextToken();
        if (token instanceof PunctuationToken) {
            if (!token.getText().equals(text)) {
                throw new ParseTreeException(error, message, token);
            }
        } else {
            // Didn't get punctuation, so we have a serious problem
            throw new ParseTreeException(error, message, token);
        }
    }
}
