/*
 * ProgramNode.java
 *
 * Created on 13 June 2006, 14:54
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.agentfactory.afapl2.compiler.parser.node;

import java.util.ArrayList;
import java.util.List;

import com.agentfactory.afapl2.compiler.parser.LexicalAnalyser;
import com.agentfactory.afapl2.compiler.parser.LexicalAnalyserException;
import com.agentfactory.afapl2.compiler.parser.ParseTreeException;
import com.agentfactory.afapl2.compiler.parser.ParserHelper;
import com.agentfactory.afapl2.compiler.parser.token.PunctuationToken;
import com.agentfactory.afapl2.compiler.parser.token.Token;
import com.agentfactory.logic.lang.FOS;
/**
 *
 * @author Administrator
 */
public class ActivityNode extends Node {
    
    public static ActivityNode create(Token token, LexicalAnalyser analyser) throws LexicalAnalyserException, ParseTreeException {
        if (token instanceof PunctuationToken) {
            throw new ParseTreeException("Syntax Error in Predicate Definition", "expected predicate but got", token);
        }
        
        ActivityNode node = new ActivityNode(token.getText());
        node.construct(analyser);
        node.setInitialToken(token);
        return node;
    }
    
    private String functor;
    private List<String> arguments;
    private List<String> types;
    
    /** Creates a new instance of ProgramNode */
    public ActivityNode(String functor) {
        this.functor = functor;
        this.arguments = new ArrayList<String>();
        this.types = new ArrayList<String>();
    }

    @Override
	public void construct(LexicalAnalyser analyser) throws LexicalAnalyserException, ParseTreeException {
        // Syntax: <predicate>(<variable>[, <variable>]*)

        // Step 1: Check if we have an open bracket "("
        Token token = analyser.getNextToken();
        if (token.getText().equals("{")) {
            analyser.back(token);
            return;
        }
        
        // Step 2: Get the first variable definition
        addVariableToTerm(analyser);
        
        token = analyser.getNextToken();
        while (token.getText().equals(",")) {
            addVariableToTerm(analyser);
            token = analyser.getNextToken();
        }
        analyser.back(token);
        ParserHelper.punctuationCheck(")", "Syntax Error in Term Definition for: " + functor, "expected ) but got", analyser);
    }

    @Override
    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append(functor);
        buf.append("(");
        boolean first = true;
        int i=0;
        while (i<arguments.size()) {
            if (first) {
                first = false;
            } else {
                buf.append(",");
            }
            buf.append(types.get(i) + ":" + arguments.get(i++));
        }
        buf.append(")");
        return buf.toString();
    }
    
    public FOS toFOS() {
        StringBuffer buf = new StringBuffer();
        buf.append(functor);
        buf.append("(");
        boolean first = true;
        int i=0;
        while (i<arguments.size()) {
            if (first) {
                first = false;
            } else {
                buf.append(",");
            }
            buf.append(arguments.get(i++));
        }
        buf.append(")");
        return new FOS(buf.toString());
    }
    
    private void addVariableToTerm(LexicalAnalyser analyser) throws LexicalAnalyserException, ParseTreeException {
        Token token = analyser.getNextToken();
        if (token.getText().startsWith("?")) {
            arguments.add(token.getText());
            types.add("none");
        } else {
            ParserHelper.punctuationCheck(":", "Syntax Error in Term Definition for: " + functor + " (..." + token.getText() +"...)", "expected : but got", analyser);
            
            String type = token.getText();
            token = analyser.getNextToken();
            if (token.getText().startsWith("?")) {
                arguments.add(token.getText());
                types.add(type);
            } else {
                throw new ParseTreeException("Syntax Error in Term Definition for: " + functor, "expected ? but found", token);
            }
        }
    }
    
    @Override
    public boolean equals(Object obj) {
        if (!(obj instanceof FOSNode)) {
            return false;
        }
        
        FOSNode fos = (FOSNode) obj;
        if (!this.functor.equals(fos.getFunctor())) {
            return false;
        }
        
        if (this.arguments.size() != fos.getArguments().size()) {
            return false;
        }

        return true;
    }
    
    public String getFunctor() {
        return functor;
    }
    
    public List<String> getVariables() {
        return arguments;
    }
    
    public List<String> getTypes() {
        return types;
    }
    
    public int numArguments() {
        return arguments.size();
    }
    
    public String getType(int index) {
        return types.get(index);
    }
    
    public String getVariable(int index) {
        return arguments.get(index);
    }
}
