/*
 * ProgramNode.java
 *
 * Created on 13 June 2006, 14:54
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.agentfactory.afapl2.compiler.parser.node;

import com.agentfactory.afapl2.compiler.parser.LexicalAnalyser;
import com.agentfactory.afapl2.compiler.parser.LexicalAnalyserException;
import com.agentfactory.afapl2.compiler.parser.ParseTreeException;
import com.agentfactory.afapl2.compiler.parser.ParserHelper;
import com.agentfactory.afapl2.compiler.parser.token.Token;

/**
 *
 * @author Administrator
 */
public class CommitNode extends Node {
    private FOSNode name;
    private FOSNode time;
    private AndNode condition;
    private FOSNode activity;
    
    /** Creates a new instance of ProgramNode */
    public CommitNode() {
    }
    
    @Override
	public void construct(LexicalAnalyser analyser) throws LexicalAnalyserException, ParseTreeException {
        // COMMIT construct syntax: COMMIT(<fosNode>, <fosNode>, <andNode>, <fosNode>)
        
        // Step 1: Check for open bracket "("
        ParserHelper.punctuationCheck("(", "Syntax Error in COMMIT", "expected ( but found", analyser);
        
        // Step 2: Get the name
        Token token = analyser.getNextToken();
        name = FOSNode.createFOS(token, analyser);

        // Step 3: Check for the first comma
        ParserHelper.punctuationCheck(",", "Syntax Error in COMMIT(" + name, "expected , but found", analyser);
        
        // Step 4: Get the time
        token = analyser.getNextToken();
        time = FOSNode.createFOS(token, analyser);

        // Step 5: Check for the second comma
        ParserHelper.punctuationCheck(",", "Syntax Error in COMMIT(" + name + "," + time, "expected , but found", analyser);
        
        // Step 6: Get the persistance condition
        token = analyser.getNextToken();
        condition = (AndNode) ParserHelper.checkForLogicOperators(",", Node.create(token, analyser), token, analyser);

        // Step 7: Check for the third comma
        ParserHelper.punctuationCheck(",", "Syntax Error in COMMIT(" + name + "," + time + "," + condition, "expected , but found", analyser);
        
        // Step 8: Get the time
        token = analyser.getNextToken();
        activity = FOSNode.createFOS(token, analyser);
        
        // Step 9: Check for an close bracket ")
        ParserHelper.punctuationCheck(")", "Syntax Error in COMMIT(" + name + "," + time + "," + condition + "," + activity, "expected ) but found", analyser);
    }
    
    public FOSNode getActivity() {
        return activity;
    }
    
    public AndNode getCondition() {
        return condition;
    }
    
    public FOSNode getName() {
        return name;
    }
    
    public FOSNode getTime() {
        return time;
    }
    
    @Override
	public String toString() {
        return "COMMIT(" + name.toString() + "," + time.toString() + "," + condition.toString() + "," + activity.toString() + ")";
    }
}
