/*
 * ProgramNode.java
 *
 * Created on 13 June 2006, 14:54
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.agentfactory.afapl2.compiler.parser.node;

import java.util.ArrayList;
import java.util.List;

import com.agentfactory.afapl2.compiler.parser.LexicalAnalyser;
import com.agentfactory.afapl2.compiler.parser.LexicalAnalyserException;
import com.agentfactory.afapl2.compiler.parser.ParseTreeException;
import com.agentfactory.afapl2.compiler.parser.ParserHelper;
import com.agentfactory.afapl2.compiler.parser.token.PunctuationToken;
import com.agentfactory.afapl2.compiler.parser.token.Token;
/**
 *
 * Syntax: FUNCTION <functor>(<term>[,<term>]) => ClassName;
 * 
 * Purpose: Defines functional terms within the ontology and maps them to
 *          classes that provide object represntations of those terms.
 * 
 * @author Rem Collier
 */
public class FunctionNode extends Node {
    
    public static FunctionNode create(Token token, LexicalAnalyser analyser) throws LexicalAnalyserException, ParseTreeException {
        if (token instanceof PunctuationToken) {
            throw new ParseTreeException("Syntax Error in FUNCTION Definition", "expected predicate but got", token);
        }
        
        FunctionNode node = new FunctionNode(token.getText());
        node.construct(analyser);
        node.setInitialToken(token);
        return node;
    }
    
    private String functor;
    private List<String> arguments;
    private List<String> types;
    private String implementation;
    
    /** Creates a new instance of ProgramNode */
    public FunctionNode(String functor) {
        this.functor = functor;
        this.arguments = new ArrayList<String>();
        this.types = new ArrayList<String>();
    }

    @Override
	public void construct(LexicalAnalyser analyser) throws LexicalAnalyserException, ParseTreeException {
        // Syntax: <predicate>(<variable>[, <variable>]*)

        // Step 1: Check if we have an open bracket "("
        Token token = analyser.getNextToken();
        if (token.getText().equals(";")) {
            analyser.back(token);
            return;
        }
        
        if (!token.getText().equals("(")) {
            throw new ParseTreeException("Syntax Error in Term Definition for: '" + functor + "'", "expected ( but found", token);
        }
        
        // Step 2: Get the first variable definition
        addArgument(analyser);
        
        token = analyser.getNextToken();
        while (token.getText().equals(",")) {
            addArgument(analyser);
            token = analyser.getNextToken();
        }
        
        analyser.back(token);
        ParserHelper.punctuationCheck(")", "Syntax Error in Term Definition for: " + functor, "expected ) but got", analyser);
        
        token = analyser.getNextToken();
        if (token.getText().equals("=>")) {
            token = analyser.getNextToken();
            implementation = token.getText();
        } else {
            analyser.back(token);
        }
    }

    @Override
    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append(functor);
        buf.append("(");
        boolean first = true;
        int i=0;
        while (i<arguments.size()) {
            if (first) {
                first = false;
            } else {
                buf.append(",");
            }
            buf.append(types.get(i) + ":" + arguments.get(i++));
        }
        buf.append(")");
        
        if (implementation != null) {
            buf.append(" => " + implementation);
        }
        return buf.toString();
    }
    
    private void addArgument(LexicalAnalyser analyser) throws LexicalAnalyserException, ParseTreeException {
        Token token = analyser.getNextToken();
        if (token.getText().startsWith("?")) {
            arguments.add(token.getText());
            types.add("none");
        } else {
            ParserHelper.punctuationCheck(":", "Syntax Error in Term Definition for: " + functor + " (..." + token.getText() +"...)", "expected : but got", analyser);
            
            String type = token.getText();
            token = analyser.getNextToken();
            if (token.getText().startsWith("?")) {
                arguments.add(token.getText());
                types.add(type);
            } else {
                throw new ParseTreeException("Syntax Error in Term Definition for: " + functor, "expected ? but found", token);
            }
        }
    }
    
    public String getFunctor() {
        return functor;
    }
    
    public List<String> getVariables() {
        return arguments;
    }
    
    public List<String> getTypes() {
        return types;
    }
    
    public String getImplementation() {
        return implementation;
    }
}
