/*
 * ProgramNode.java
 *
 * Created on 13 June 2006, 14:54
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.agentfactory.afapl2.compiler.parser.node;

import com.agentfactory.afapl2.compiler.parser.LexicalAnalyser;
import com.agentfactory.afapl2.compiler.parser.LexicalAnalyserException;
import com.agentfactory.afapl2.compiler.parser.ParseTreeException;
import com.agentfactory.afapl2.compiler.parser.ParserHelper;
import com.agentfactory.afapl2.compiler.parser.token.Token;

/**
 *
 * @author Administrator
 */
public class PlanNode extends Node {
    private ActivityNode identifier;
    private Node preCondition;
    private Node postCondition;
    private FOSNode body;
    
    /** Creates a new instance of ProgramNode */
    public PlanNode() {
    }
    
    @Override
	public void construct(LexicalAnalyser analyser) throws LexicalAnalyserException, ParseTreeException {
        // ACTION construct syntax:
        // PLAN <fosNode> {
        //     PRECONDITION <node>;
        //     POSTCONDITION <node>;
        //     BODY <planOperator> | <fosNode>;
        // }
        
        // Step 1: Extract identifier
        Token token = analyser.getNextToken();
        identifier = ActivityNode.create(token, analyser);
        
        // Step 2: Check that we have an open brace '{'
        ParserHelper.punctuationCheck("{", "Syntax Error in ACTION " + identifier, "expected { but got", analyser);
        
        // Step 3: Extract precondition, postcondition, and class
        token = analyser.getNextToken();
        while (!token.getText().equals("}")) {
            if (token.getText().equals("PRECONDITION")) {
                preCondition = ParserHelper.checkForLogicOperators(";", Node.create(analyser.getNextToken(), analyser), token, analyser);
                ParserHelper.punctuationCheck(";", "Syntax Error in PRECONDITION parameter of PLAN " + identifier, "expected ; but got", analyser);
            } else if (token.getText().equals("POSTCONDITION")) {
                postCondition = ParserHelper.checkForLogicOperators(";", Node.create(analyser.getNextToken(), analyser), token, analyser);
                ParserHelper.punctuationCheck(";", "Syntax Error in POSTCONDITION parameter of PLAN " + identifier, "expected ; but got", analyser);
            } else if (token.getText().equals("BODY")) {
                body = FOSNode.createFOS(analyser.getNextToken(), analyser);
                ParserHelper.punctuationCheck(";", "Syntax Error in BODY parameter of PLAN " + identifier, "expected ; but got", analyser);
            } else {
                throw new ParseTreeException("Syntax Error in ACTION " + identifier, "expected PRECONDITION, POSTCONDITION, or CLASS but got", token);
            }
            
            token = analyser.getNextToken();
        }        
        
        if (preCondition == null) {
            preCondition = new FOSNode("true");
        }
        
        if (postCondition == null) {
            postCondition = new FOSNode("true");
        }
        if (body == null) {
            throw new LexicalAnalyserException(analyser.getSource(), token.getLine(), "ACTION " + identifier + " is missing required CLASS parameter");
        }

        token = analyser.getNextToken();
        if (token != null) {
            analyser.back(token);
        }
        
        if ((token == null) || !token.getText().equals(";")) {
            Token temp = Token.createToken(";", -1, null);
            analyser.back(temp);
        }
    }
    
    
    @Override
	public String toString() {
        return "PLAN " + identifier + " { PRECONDITION " + preCondition +
               "; POSTCONDITION " + postCondition + "; BODY " + body + "; }";
    }
    
    public FOSNode getPlan() {
        return body;
    }
    
    public ActivityNode getIdentifier() {
        return identifier;
    }
    
    public Node getPreCondition() {
        return preCondition;
    }

    public Node getPostCondition() {
        return postCondition;
    }
}
