/*
 * ProgramNode.java
 *
 * Created on 13 June 2006, 14:54
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.agentfactory.afapl2.compiler.parser.node;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.afapl2.compiler.parser.LexicalAnalyser;
import com.agentfactory.afapl2.compiler.parser.LexicalAnalyserException;
/**
 *
 * @author Administrator
 */
public class ProgramNode extends Node {
    private String source;
    private List<ImpliesNode> beliefRules;
    private List<ImpliesNode> commitmentRules;
    private List<ModalNode> beliefs;
    private List<GoalNode> goals;
    private List<CommitNode> commitments;
    private List<PerceptorNode> perceptors;
    private List<ActionNode> actions;
    private List<ModuleNode> modules;
    private List<ImportNode> imports;
    private List<RoleNode> roles;
    private List<OntologyNode> ontologies;
    private List<PlanNode> plans;
    
    /** Creates a new instance of ProgramNode */
    public ProgramNode(String source) {
        this.source = source;
        
        beliefRules = new LinkedList<ImpliesNode>();
        commitmentRules = new LinkedList<ImpliesNode>();
        beliefs = new LinkedList<ModalNode>();
        goals = new LinkedList<GoalNode>();
        commitments = new LinkedList<CommitNode>();
        perceptors = new LinkedList<PerceptorNode>();
        actions = new LinkedList<ActionNode>();
        modules = new LinkedList<ModuleNode>();
        imports = new LinkedList<ImportNode>();
        roles = new LinkedList<RoleNode>();
        ontologies = new LinkedList<OntologyNode>();
        plans = new LinkedList<PlanNode>();
    }
    
    @Override
	public void construct(LexicalAnalyser analyser) throws LexicalAnalyserException {
        // DO NOTHING - This is a root node that is always created...
    }
    
    @Override
	public boolean addChild(Node node) {
        if (node instanceof ImpliesNode) {
            ImpliesNode iNode = (ImpliesNode) node;
            if (iNode.getRHS() instanceof BeliefNode) {
                beliefRules.add((ImpliesNode) node);
            } else if (iNode.getRHS() instanceof CommitNode) {
                commitmentRules.add((ImpliesNode) node);
            } else {
                return false;
            }
        } else if (node instanceof ModalNode) {
            if (node instanceof GoalNode) {
                goals.add((GoalNode) node);
            } else {
                beliefs.add((ModalNode) node);
            }
        } else if (node instanceof CommitNode) {
            commitments.add((CommitNode) node);
        } else if (node instanceof PerceptorNode) {
            perceptors.add((PerceptorNode) node);
        } else if (node instanceof ActionNode) {
            actions.add((ActionNode) node);
        } else if (node instanceof ModuleNode) {
            modules.add((ModuleNode) node);
        } else if (node instanceof ImportNode) {
            imports.add((ImportNode) node);
        } else if (node instanceof RoleNode) {
            roles.add((RoleNode) node);
        } else if (node instanceof OntologyNode) {
            ontologies.add((OntologyNode) node);
        } else if (node instanceof PlanNode) {
            plans.add((PlanNode) node);
        } else {
            System.out.println("FAILED TO ADD: " + node);
        }
        
        return true;
    }
    
    public List<PerceptorNode> getPerceptors() {
        return perceptors;
    }
    
    public List<ActionNode> getActions() {
        return actions;
    }
    
    public List<ModuleNode> getModules() {
        return modules;
    }
    
    public List<ImportNode> getImports() {
        return imports;
    }
    
    @Override
	public String toString() {
        return source;
    }
    
    public List<ImpliesNode> getCommitmentRules() {
        return commitmentRules;
    }
    
    public List<PlanNode> getPlans() {
        return plans;
    }
    
    public List<OntologyNode> getOntologies() {
        return ontologies;
    }
    
    public List<ModalNode> getBeliefs() {
        return beliefs;
    }
    
    public List<GoalNode> getGoals() {
        return goals;
    }
    
    public List<CommitNode> getCommitments() {
        return commitments;
    }
    
    public List<ImpliesNode> getBeliefRules() {
        return beliefRules;
    }
    
    public List<RoleNode> getRoles() {
        return roles;
    }
    
    @Override
	public String getSource() {
        return source;
    }
}
