/*
 * ProgramNode.java
 *
 * Created on 13 June 2006, 14:54
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.agentfactory.afapl2.compiler.parser.node;

import com.agentfactory.afapl2.compiler.parser.LexicalAnalyser;
import com.agentfactory.afapl2.compiler.parser.LexicalAnalyserException;
import com.agentfactory.afapl2.compiler.parser.ParseTreeException;
import com.agentfactory.afapl2.compiler.parser.token.PunctuationToken;
import com.agentfactory.afapl2.compiler.parser.token.Token;
/**
 *
 * Syntax: TYPEDEF <identifier> <type>
 * 
 * Purpose: To allow meaningful types to be defined that are based on existing
 *          types.
 * 
 * @author Rem Collier
 */
public class TypeDefNode extends Node {
    
    public static TypeDefNode create(Token token, LexicalAnalyser analyser) throws LexicalAnalyserException, ParseTreeException {
        if (token instanceof PunctuationToken) {
            throw new ParseTreeException("Syntax Error in Predicate Definition", "expected predicate but got", token);
        }
        
        TypeDefNode node = new TypeDefNode(token.getText());
        node.construct(analyser);
        node.setInitialToken(token);
        return node;
    }
    
    private String identifier;
    private String type;
    
    /** Creates a new instance of ProgramNode */
    public TypeDefNode(String identifier) {
        this.identifier = identifier;
    }

    @Override
	public void construct(LexicalAnalyser analyser) throws LexicalAnalyserException, ParseTreeException {
        // Syntax: <identifier> <type>

        // Step 1: Check if we have an open bracket "{"
        Token token = analyser.getNextToken();
        if (token.getText().startsWith("?")) {
            throw new ParseTreeException("Syntax Error in TYPEDEF definition for: '" + identifier + "'", "variables not permitted, but found", token);
        }
        
        // Step 2: Get the first variable definition
        type = token.getText();
    }

    @Override
    public String toString() {
        return "TYPEDEF " + identifier + " " + type;
    }
        
    public String getIdentifier() {
        return identifier;
    }
    
    public String getType() {
        return type;
    }
}
