/**
 * Title:       FileReader.java
 * Copyright:   Copyright (c) 1996-2004 The Agent Factory Working Group. All rights reserved.
 * Licence:     This file is free software; you can redistribute it and/or modify
 *              it under the terms of the GNU Lesser General Public License as published by
 *              the Free Software Foundation; either version 2.1, or (at your option)
 *              any later version.
 *
 *              This file is distributed in the hope that it will be useful,
 *              but WITHOUT ANY WARRANTY; without even the implied warranty of
 *              MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *              GNU Lesser General Public License for more details.
 *
 *              You should have received a copy of the GNU Lesser General Public License
 *              along with Agent Factory; see the file COPYING.  If not, write to
 *              the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *              Boston, MA 02111-1307, USA.
 */

package com.agentfactory.afapl2.compiler.parser.reader;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.LinkedList;
import java.util.List;
/**
 * This class implements the Reader interface to support reading of characters
 * from strings.  It is one of a number of reader classes that may be used to
 * compile AF-APL code.
 *
 * @author  Rem Collier
 */
public class InputStreamReader implements Reader {
    private BufferedReader reader;
    private List<Character> buffer;
    private InputStream stream;
    
    /**
     * Creates a new instance of StringReader
     * @param code the code to be compiled
     */
    public InputStreamReader(InputStream stream) {
    	this.stream = stream;
        buffer = new LinkedList<Character>();
        reader = new BufferedReader(new java.io.InputStreamReader(stream));
    }
    
    /**
     * gets the next character from the reader.
     * @return the next character or '\0'
     */
    public char nextCharacter() {
        char ch = '\0';
        
        if (buffer.size() > 0) {
            ch = buffer.remove(0);
        } else {
            try {
                int value = reader.read();
                
                if (value >= 0) {
                    ch = (char) value;
                } else {
                    ch = '\0';
                }
                
            } catch (IOException ioe) {
                System.out.println("error reading from file: " + ioe);
                ch = '\0';
            }
        }
        
        return ch;
    }
    
    /**
     * used to undo the nextCharacter method.
     * @param ch the character to be put back into the reader.
     */
    public void pushBack(char ch) {
        buffer.add(0, ch);
    }
    
    public void close() {
        try {
            stream.close();
        } catch (IOException ioe) {
            System.out.println("could not close file");
        }
    }
}
