/**
 * Title:       Reader.java
 * Copyright:   Copyright (c) 1996-2004 The Agent Factory Working Group. All rights reserved.
 * Licence:     This file is free software; you can redistribute it and/or modify
 *              it under the terms of the GNU Lesser General Public License as published by
 *              the Free Software Foundation; either version 2.1, or (at your option)
 *              any later version.
 *
 *              This file is distributed in the hope that it will be useful,
 *              but WITHOUT ANY WARRANTY; without even the implied warranty of
 *              MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *              GNU Lesser General Public License for more details.
 *
 *              You should have received a copy of the GNU Lesser General Public License
 *              along with Agent Factory; see the file COPYING.  If not, write to
 *              the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *              Boston, MA 02111-1307, USA.
 */

package com.agentfactory.afapl2.compiler.parser.reader;

/**
 * Interface for the reader component of the AF-APL compiler.  This is used
 * to facilitate the implementation of a compiler that is able to compile
 * AF-APL code from a number of sources.
 *
 * @author  Rem Collier
 */
public interface Reader {
    /**
     * gets the next character from the reader.
     * @return the next character or '\0'
     */
    public char nextCharacter();
    
    /**
     * used to undo the nextCharacter method.
     * @param ch the character to be put back into the reader.
     */
    public void pushBack(char ch);
    
    public void close();
}
