/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package com.agentfactory.afapl2.compiler.typeCheck;

import java.util.HashMap;
import java.util.Map;

import com.agentfactory.afapl2.compiler.parser.node.FOSNode;

/**
 *
 * @author remcollier
 */
public class BasicTypeChecker {
    protected Map<String, ITypeChecker> checkers;
    
    public BasicTypeChecker() {
        checkers = new HashMap<String, ITypeChecker>();
        checkers.put("string", new StringTypeChecker());
        checkers.put("short", new ShortTypeChecker());
        checkers.put("int", new IntTypeChecker());
        checkers.put("long", new LongTypeChecker());
        checkers.put("float", new FloatTypeChecker());
        checkers.put("double", new DoubleTypeChecker());
        checkers.put("char", new CharTypeChecker());
        checkers.put("boolean", new BooleanTypeChecker());
    }

    public boolean validateTerm(String type, FOSNode fos) {
        // Default Type: maintained for backwards compatibility with earlier
        // versions of AFAPL2
        if (type.equals("none")) return true;
        
        if (type.startsWith("sequence")) {
            return validateSequence(type.substring(9, type.length()-1), fos);
        }
        
        if (type.startsWith("set")) {
            return validateSequence(type.substring(4, type.length()-1), fos);
        }
        
        // Now do the real type checking...
        ITypeChecker typeChecker = checkers.get(type);
        if (typeChecker == null) {
            System.out.println("No type specified for: " + type + ": " + fos);
            return false;
        }
        
        return typeChecker.validate(fos);
    }
    
    private boolean validateSequence(String type, FOSNode fos) {
        for (FOSNode arg: fos.getArguments()) {
            if (!validateTerm(type, arg)) return false;
        }
        return true;
        
    }
}
