package com.agentfactory.afapl2.core.actuator;

import com.agentfactory.logic.agent.Actuator;
import com.agentfactory.logic.agent.Belief;
import com.agentfactory.logic.lang.FOS;
import com.agentfactory.service.ams.AgentManagementService;

/**
 * This actuator can be used to suspend agents at run-time. To use it, the
 * agent must first be bound to the AgentManagementService platform service.
 * 
 * The corresponding action must take one parameter:
 * <ul>
 * <li>name: the name of the agent
 * </ul>
 * 
 * @author  remcollier
 */
public class SuspendAgent extends Actuator {

    @Override
	public boolean act(FOS action) {
        String name = action.argAt(0).toString();
        
        AgentManagementService ams = (AgentManagementService) getService(AgentManagementService.NAME);
        if (ams == null) {
            adoptBelief("BELIEF(notBoundToService(" + AgentManagementService.NAME + "))");
            return false;
        }
        if (ams.suspendAgent(name)) {
            Belief belief = createBelief("suspendedAgent");
            belief.bind("?name", name);
            adoptBelief(belief);
            return true;
        }
        
        Belief belief = createBelief("unknownAgent");
        belief.bind("?name", name);
        adoptBelief(belief);
        return false;
    }
}
