/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.agentfactory.afapl2.debugger.inspector;

import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.MouseListener;
import java.util.Enumeration;
import java.util.Iterator;

import javax.swing.DefaultListModel;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JToolBar;
import javax.swing.JTree;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;

import com.agentfactory.afapl2.debugger.AFAPL2Snapshot;
import com.agentfactory.afapl2.debugger.model.CommitmentDescription;
import com.agentfactory.visualiser.core.ISnapshot;
import com.agentfactory.visualiser.impl.InspectorPanel;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 *
 * @author remcollier
 */
public class CommitmentsPanel extends JPanel implements InspectorPanel {
    private JList list;
    private JComboBox filterBox;
    private AFAPL2Snapshot snapshot;
    private JTree commitmentTree;

    public CommitmentsPanel() {
        this.setLayout(new BorderLayout());

        // Set up the filter
        JToolBar toolbar = new JToolBar();
        toolbar.add(new JLabel("Filter:"));
        filterBox = new JComboBox();
        filterBox.addItem("--All--");
        filterBox.addItem("Primary Commitments");
        filterBox.addActionListener(new ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent event) {
                update();
            }
        });
        toolbar.add(filterBox);
        this.add(BorderLayout.NORTH, toolbar);

        JSplitPane splitPane = new JSplitPane();
        splitPane.setOrientation(JSplitPane.VERTICAL_SPLIT);
        add(BorderLayout.CENTER, splitPane);

        list = new JList(new DefaultListModel());
        list.addMouseListener(new MouseListener() {
            public void mouseClicked(java.awt.event.MouseEvent mouseEvent) {
                DefaultTreeModel model = (DefaultTreeModel) commitmentTree.getModel();
                if (list.getSelectedValue() == null) {
                    model.setRoot(new DummyNode("NO_COMMITMENT_SELECTED"));
                } else {
                    model.setRoot(new CommitmentTreeNode((CommitmentDescription) list.getSelectedValue()));
                }
            }

            public void mouseReleased(java.awt.event.MouseEvent mouseEvent) {
            }

            public void mousePressed(java.awt.event.MouseEvent mouseEvent) {
            }

            public void mouseExited(java.awt.event.MouseEvent mouseEvent) {
            }

            public void mouseEntered(java.awt.event.MouseEvent mouseEvent) {
            }
        });
        JScrollPane pane = new JScrollPane(list);
        splitPane.setTopComponent(pane);
        splitPane.setDividerLocation(100);

        commitmentTree = new JTree(new DefaultTreeModel(new DummyNode("NO_COMMITMENT_SELECTED")));
        pane = new JScrollPane(commitmentTree);
        splitPane.setBottomComponent(pane);
    }

    public void update(ISnapshot snapshot) {
        this.snapshot = (AFAPL2Snapshot) snapshot;
        update();
    }
    
    public void setCommitment(CommitmentDescription commitment) {
        list.setSelectedValue(commitment, true);
        DefaultTreeModel model = (DefaultTreeModel) commitmentTree.getModel();
        model.setRoot(new CommitmentTreeNode((CommitmentDescription) list.getSelectedValue()));
    }

    public synchronized void update() {
        CommitmentDescription selection = (CommitmentDescription) list.getSelectedValue();
        DefaultListModel model = (DefaultListModel) list.getModel();

        model.clear();
        if (filterBox.getSelectedItem().equals("--All--")) {
            Iterator it = snapshot.getCommitments().iterator();
            while (it.hasNext()) {
                model.addElement(it.next());
            }
        } else {
            for (CommitmentDescription commitment : snapshot.getPrimaryCommitments()) {
                model.addElement(commitment);
            }
        }

        if (selection != null) {
            CommitmentTreeNode node = (CommitmentTreeNode) commitmentTree.getModel().getRoot();
            DefaultTreeModel treeModel = (DefaultTreeModel) commitmentTree.getModel();
            
            if (model.contains(selection)) {
                node.rebuild();
                list.setSelectedValue(selection, true);
            } else {
                treeModel.setRoot(new DummyNode("NO_COMMITMENT_SELECTED"));
            }
            commitmentTree.invalidate();
        }
        list.validate();
    }

    public class DummyNode implements TreeNode {
        private String text;

        /** Creates a new instance of PlatformsNode */
        public DummyNode(String _text) {
            text = _text;
        }

        public Enumeration children() {
            return null;
        }

        public TreeNode getParent() {
            return null;
        }

        public boolean isLeaf() {
            return true;
        }

        public boolean getAllowsChildren() {
            return false;
        }

        public int getIndex(TreeNode node) {
            return -1;
        }

        public int getChildCount() {
            return 0;
        }

        public TreeNode getChildAt(int index) {
            return null;
        }

        @Override
		public String toString() {
            return text;
        }
    }

    private class CommitmentTreeNode implements TreeNode {
        CommitmentTreeNode parent;
        CommitmentDescription commitment;
        List<CommitmentTreeNode> children;

        public CommitmentTreeNode(CommitmentTreeNode parent, CommitmentDescription commitment) {
            this.parent = parent;
            this.commitment = commitment;
            rebuild();
        }

        public void rebuild() {
            children = new ArrayList<CommitmentTreeNode>();
            
            for (int i: commitment.getChildren()) {
                children.add(new CommitmentTreeNode(this, snapshot.getCommitments().get(i)));
            }
        }

        public CommitmentTreeNode(CommitmentDescription commitment) {
            this(null, commitment);
        }

        public Enumeration children() {
            return Collections.enumeration( children );
        }

        public boolean isLeaf() {
            return children.isEmpty();
        }

        public boolean getAllowsChildren() {
            return true;
        }

        public int getIndex(TreeNode node) {
            return children.indexOf(node);
        }

        public TreeNode getParent() {
            return parent;
        }

        public int getChildCount() {
            return children.size();
        }

        public TreeNode getChildAt(int i) {
            return (TreeNode) children.get(i);
        }

        @Override
        public String toString() {
            StringBuffer buf = new StringBuffer();
            buf.append("COMMIT(");
            buf.append(commitment.getTo());
            buf.append(",");
            buf.append(commitment.getWhen());
            buf.append(",");
            buf.append(commitment.getMaintenance());
            buf.append(",");
            buf.append(commitment.getActivity());
            buf.append(")");
            return buf.toString();
        }

//        public int getIndex() {
//            return snapshot.getCommitments().indexOf(commitment);
//        }
    }
}
