package com.agentfactory.afapl2.interpreter;

import java.util.List;
import java.util.Set;

import com.agentfactory.afapl2.interpreter.mentalState.Goal;
import com.agentfactory.logic.lang.FOS;
import com.agentfactory.logic.reasoner.Queryable;

public interface IGoalManager extends Queryable {
    /**
     * Returns a list of goals
     * 
     * @return the list of goals
     */
    public Set<Goal> getGoals();
    
    /**
     * Add a new goal.  The goal will only be added in the event that it is
     * not already a goal.
     * 
     * @param _goal the goal to be added
     * 
     * @return true if the goal is added, false otherwise
     */
    public Goal adoptGoal(String goal);
    
    /**
     * Add a new goal.  The goal will only be added in the event that it is
     * not already a goal.
     * 
     * @param _goal the goal to be added
     * 
     * @return true if the goal is added, false otherwise
     */
    public Goal mantainGoal(String goal);
    
    /**
     * Add a new goal.  The goal will only be added in the event that it is
     * not already a goal.
     * 
     * @param _goal the goal to be added
     * 
     * @return true if the goal is added, false otherwise
     */
    public Goal adoptGoal(Goal goal);
    
    public boolean dropGoal(String goal);
    
    /**
     * Required by the Queryable interface and used to return a list of
     * FOS objects that match the given query.
     * 
     * @param fos the fos query
     * @return a list of matching fos objects
     */
    public List<Object> query(FOS fos);
    
    /**
     * Manage Goals: called on each iteration of the agent.
     */
    public void manage();    
    
    
    
}
