/*
 * RoleLibrary.java
 *
 * Created on 29 January 2005, 14:26
 */
package com.agentfactory.afapl2.interpreter.control;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import com.agentfactory.afapl2.interpreter.AbstractAFAPL2Agent;
import com.agentfactory.afapl2.interpreter.mentalState.Role;
import com.agentfactory.logic.lang.FOS;
import com.agentfactory.logic.reasoner.Substitution;
import com.agentfactory.logic.reasoner.SubstitutionSet;

/**
 *
 * @author  Administrator
 */
public class RoleLibrary {
    private Map<String, RoleTemplate> roles;
    private List<Role> instances;
    private AbstractAFAPL2Agent agent;
    
    /** Creates a new instance of RoleLibrary */
    public RoleLibrary(AbstractAFAPL2Agent agent) {
        this.agent = agent;
        roles = new HashMap<String, RoleTemplate>();
        instances = new LinkedList<Role>();
    }

    public void addRoleTemplate(RoleTemplate role) {
        String roleId = role.getIdentifier().getFunctor();
        roles.put(roleId, role);
    }
    
    protected RoleTemplate getRoleTemplate(FOS identifier) {
        RoleTemplate role = roles.get(identifier.getFunctor());
        if ((role == null) || (!role.getIdentifier().matches(identifier))) return null;
        return role;
    }
    
    public boolean enactRole(FOS identifier) {
        if (isEnacted(identifier)) return false;
        
        RoleTemplate template = getRoleTemplate(identifier);
        if (template == null) return false;
        
        // Okay, so we have a role to instantiate - start by creating a
        // variable binding for the role
        SubstitutionSet binding = new SubstitutionSet();
        for (int i=0; i < identifier.numArguments(); i++) {
            binding.addBinding(new Substitution(template.getIdentifier().copy().argAt(i),
                                                identifier.copy().argAt(i)));
        }
        
        Role role = template.instantiate(binding, this);
        if (role == null) return false;
        
        instances.add(role);
        
        return true;
    }
    
    public boolean deactRole(FOS identifier) {
        Role role = getInstance(identifier);
        if (role == null) return false;
        instances.remove(role);
        RoleTemplate template = getRoleTemplate(identifier);
        template.destroy(role);
        return true;
    }
    
    public List<Role> getInstances() {
        return instances;
    }
    
    public boolean isEnacted(FOS identifier) {
        return getInstance(identifier) != null;
    }
    
    private Role getInstance(FOS identifier) {
        for (Role role: instances) {
            if (role.getIdentifier().equals(identifier)) {
                return role;
            }
        }
        return null;
    }
    
    public Collection<RoleTemplate> getRoles() {
        return roles.values();
    }
    
    public void enactRoles() {
        List<Object> triggers = null;
        List<FOS> variables = null;
        SubstitutionSet keySet = null;
        FOS variable = null, constant = null;
        Role role = null;
        
        for (RoleTemplate template: roles.values()) {
            variables = template.getIdentifier().getVariables();
            
            triggers = template.getTriggers();
            for (int j=0; j < triggers.size(); j++) {
                List<SubstitutionSet> bindings = agent.queryAll(triggers.get(j));
                if (bindings != null) {
                    for (SubstitutionSet set: bindings) {
                        keySet = new SubstitutionSet();
                        
                        for ( int k=0; k < variables.size(); k++ ) {
                            variable = variables.get( k );
                            constant = set.findConstant( variable );
                            keySet.addBinding(new Substitution(variable, constant));
                        }
                        role = template.instantiate(keySet, this);
                        if (role != null) {
                            instances.add(role);
                        }
                    }
                }
            }
        }
    }
}
