/*
 * ActivityHandler.java
 *
 * Created on 11 November 2005, 09:50
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.agentfactory.afapl2.interpreter.control.cms;
import java.util.HashMap;

import com.agentfactory.afapl2.interpreter.AbstractAFAPL2Agent;
import com.agentfactory.afapl2.interpreter.InterpreterHelper;
import com.agentfactory.afapl2.interpreter.mentalState.Action;
import com.agentfactory.afapl2.interpreter.mentalState.Commitment;
import com.agentfactory.afapl2.interpreter.mentalState.PlanInstance;
import com.agentfactory.logic.lang.FOS;
import java.util.Map;

/**
 *
 * @author Administrator
 */
public abstract class ActivityHandler {
    private static Map<String,String> handlers;
    protected Commitment commitment;
    protected AbstractAFAPL2Agent agent;
    
    public static ActivityHandler getInstance(Commitment commitment, AbstractAFAPL2Agent agent) {
        FOS activity = commitment.getActivity();
        
        if (handlers == null) {
            handlers = new HashMap<String,String>();
            handlers.put("PAR", "com.agentfactory.afapl2.interpreter.control.cms.PARActivityHandler");
            handlers.put("SEQ", "com.agentfactory.afapl2.interpreter.control.cms.SEQActivityHandler");
            handlers.put("OR", "com.agentfactory.afapl2.interpreter.control.cms.ORActivityHandler");
            handlers.put("DO_WHEN", "com.agentfactory.afapl2.interpreter.control.cms.DOWHENActivityHandler");
            handlers.put("DO_UNTIL", "com.agentfactory.afapl2.interpreter.control.cms.DOUNTILActivityHandler");
            handlers.put("AWAIT", "com.agentfactory.afapl2.interpreter.control.cms.AWAITActivityHandler");
            handlers.put("DELAY", "com.agentfactory.afapl2.interpreter.control.cms.DELAYActivityHandler");
            handlers.put("FOREACH", "com.agentfactory.afapl2.interpreter.control.cms.FOREACHActivityHandler");
            handlers.put("ATTEMPT", "com.agentfactory.afapl2.interpreter.control.cms.ATTEMPTActivityHandler");
            handlers.put("ADOPT", "com.agentfactory.afapl2.interpreter.control.cms.ADOPTActivityHandler");
            handlers.put("RETRACT", "com.agentfactory.afapl2.interpreter.control.cms.RETRACTActivityHandler");
            handlers.put("MAINTAIN", "com.agentfactory.afapl2.interpreter.control.cms.MAINTAINActivityHandler");
        }
        
        ActivityHandler handler = null;
        
        String handlerClass = (String) handlers.get(activity.getFunctor());
        if (handlerClass != null) {
            try {
                Class myClass = Class.forName(handlerClass);
                handler = (ActivityHandler) myClass.newInstance();
                handler.setCommitment(commitment);
                handler.setAgent(agent);
            } catch (ClassNotFoundException cnfe) {
                System.out.println("[ActivityHandler] Could not find class: " + handlerClass);
            } catch (IllegalAccessException ie) {
                System.out.println("[ActivityHandler] Could not instantiate class: " + handlerClass);
            } catch (InstantiationException ie) {
                System.out.println("[ActivityHandler] Could not instantiate class: " + handlerClass);
            }
            return handler;
        }

        // Not a plan operator, so try the plan library...
        PlanInstance plan = agent.getPlanLibrary().getPlan(activity);
        if (plan != null) {
            handler = new PlanActivityHandler(plan);
            handler.setCommitment(commitment);
            handler.setAgent(agent);
            return handler;
        }

        // Finally, try actions...
        Action action = agent.getActions().get(InterpreterHelper.createSignature(activity));
        if (action != null) {
            handler = new PrimitiveActivityHandler(action);
            handler.setCommitment(commitment);
            handler.setAgent(agent);
            return handler;
        } else {
            System.out.println( "WARNING: Cound not bind activity to action/plan: " + activity.getFunctor() );
        }
        
        handler = new NoActivityHandler();
        handler.setCommitment(commitment);
        handler.setAgent(agent);
        return handler;
    }
    
    public void addCommitment(Commitment commitment) {
        agent.getCommitmentManager().addInitialCommitment(commitment);
    }
    
    public void setCommitment(Commitment _commitment) {
        commitment = _commitment;
        commitment.setHandler( this );
    }
    
    public void setAgent(AbstractAFAPL2Agent _agent) {
        agent = _agent;
    }
    
    public boolean handle() {
        if (commitment.getState() == Commitment.STATE_SUCCEEDED) {
            if (commitment.getParent() == null && commitment.getCommitmentRule() != null) {
                commitment.getCommitmentRule().removeCommitment(commitment);
                commitment.setCommitmentRule(null);
            } 
            
            return false;
        }
        
        return true;
    }
    
    public void handleEvent(int event) {
        Commitment parent = commitment.getParent();
        if (parent != null) {
            if (commitment.getState() == Commitment.STATE_SUCCEEDED) {
                parent.update(Commitment.EVENT_CHILD_SUCCEEDED);
            } else if (commitment.getState() == Commitment.STATE_FAILED) {
                parent.update(Commitment.EVENT_CHILD_FAILED);
            }
        }
    }
}
