/*
 * PARActivityHandler.java
 *
 * Created on 11 November 2005, 09:53
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.agentfactory.afapl2.interpreter.control.cms;
import com.agentfactory.afapl2.interpreter.mentalState.Commitment;
import com.agentfactory.logic.lang.FOS;
import com.agentfactory.logic.lang.TerAND;
/**
 * Usage: DO_UNTIL(<condition>, <plan>)
 * 
 * Description: When this plan operator is evaluated, the <condition> is
 *              checked. if satisfied, the <plan> is added into the commitment
 *              tree. If not satisifed, the corresponding commitment is
 *              satisfied.
 * 
 * State Transitions:  |------------------\/
 *                    NEW -> ACTIVE -> WAITING -> SUCCEEDED
 *                             /\---------|
 *
 *                    FAILURE can be received at any time upto SUCCEEDED
 * 
 * @author Rem Collier
 * @date 3rd November 2008
 */
public class DOUNTILActivityHandler extends ActivityHandler {
    
    /** Creates a new instance of PARActivityHandler */
    public DOUNTILActivityHandler() {
    }
    
    @Override
    public boolean handle() {
        Commitment newCommitment = null;
        TerAND condition = new TerAND();
        condition.add(new FOS("BELIEF(true)"));

        if ((commitment.getState() == Commitment.STATE_NEW) ||
            (commitment.getState() == Commitment.STATE_WAITING)) {
            // Check the <condition>
            FOS plan = commitment.getActivity();
            if (!agent.query(plan.argAt(0))) {
                // <condition> is satisfied
                newCommitment = new Commitment(commitment.getTo(), new FOS("?now"), condition, plan.argAt(1), commitment);
                newCommitment.evaluate();
                addCommitment(newCommitment);
                commitment.update(Commitment.EVENT_ACTIVATED);
            } else {
                 // <condition> is not satisfied
                commitment.update(Commitment.EVENT_SUCCEEDED);
           }
           return true;
        } else if (commitment.getState() == Commitment.STATE_ACTIVE) {
           if (!commitment.hasChildren()) {
               commitment.update(Commitment.EVENT_CHILD_SUCCEEDED);
           }
        } else if (commitment.getState() == Commitment.STATE_REDUNDANT) {
            return false;
        }
        
        return super.handle();
    }
    
    @Override
	public void handleEvent(int event) {
        if (event == Commitment.EVENT_ACTIVATED) {
            commitment.setState(this, Commitment.STATE_ACTIVE);
        } else if (event == Commitment.EVENT_CHILD_SUCCEEDED) {
            if (commitment.removeCompletedChildren()) {
                commitment.setState(this, Commitment.STATE_WAITING);
            }
        } else if (event == Commitment.EVENT_SUCCEEDED) {
            commitment.setState(this, Commitment.STATE_SUCCEEDED);
        } else if (event == Commitment.EVENT_CHILD_FAILED) {
            commitment.removeFailedChildren();
            commitment.setState(this, Commitment.STATE_FAILED);
            commitment.notifyChildren(Commitment.EVENT_PARENT_FAILED);
        } else if (event == Commitment.EVENT_PARENT_FAILED) {
            commitment.notifyChildren(Commitment.EVENT_PARENT_FAILED);
            commitment.setState(this, Commitment.STATE_FAILED);
        } else if (event == Commitment.EVENT_REDUNDANT) {
            commitment.notifyChildren(Commitment.EVENT_REDUNDANT);
            commitment.setState(this, Commitment.STATE_REDUNDANT);
        }
        super.handleEvent(event);
    }
}
