/*
 * PlanActivityHandler.java
 *
 * Originally Created on 11 November 2005, 09:53
 *
 */

package com.agentfactory.afapl2.interpreter.control.cms;
import com.agentfactory.afapl2.interpreter.mentalState.Commitment;
import com.agentfactory.afapl2.interpreter.mentalState.PlanInstance;
import com.agentfactory.logic.lang.FOS;
import com.agentfactory.logic.lang.TerAND;

/**
 *
 * @author The Unknown Coder
 * @version $Id$
 */
public class PlanActivityHandler extends ActivityHandler {
    private PlanInstance plan;
    
    /** Creates a new instance of PARActivityHandler */
    public PlanActivityHandler(PlanInstance plan) {
        this.plan = plan;
    }
    
    @Override
    public boolean handle() {
        Commitment newCommitment = null;
        TerAND condition = new TerAND();
        condition.add(new FOS("BELIEF(true)"));

        if (commitment.getState() == Commitment.STATE_NEW) {
            if (agent.query(plan.getPreCondition())) {
                newCommitment = new Commitment(commitment.getTo(), new FOS("?now"), condition, plan.getBody(), commitment);
                newCommitment.evaluate();
                addCommitment(newCommitment);
                commitment.update(Commitment.EVENT_ACTIVATED);
            }
            else {
                // preconditions not satisfied so the commitment fails
                commitment.update(Commitment.EVENT_FAILED);
            }
            return true;
        } else if (commitment.getState() == Commitment.STATE_ACTIVE) {
            // DO NOTHING - WAITING FOR THE PLAN TO COMPLETE
        } else if (commitment.getState() == Commitment.STATE_WAITING) {
            throw new RuntimeException("Commitment: " + commitment + "\nShould not be in state: STATE_WAITING");
        } else if (commitment.getState() == Commitment.STATE_FAILED) {
            if (!commitment.hasChildren()) {
                Commitment parent = commitment.getParent();
                if (parent != null) {
                    parent.update(Commitment.EVENT_CHILD_FAILED);
                }
                return false;
            }
        } else if (commitment.getState() == Commitment.STATE_REDUNDANT) {
            return false;
        }
        
        return super.handle();
    }
    
    @Override
	public void handleEvent(int event) {
//        if (commitment.getState() == Commitment.STATE_REDUNDANT) {
//            System.out.println("Redundant state: do nothing");
//            return;
//        }
        if (event == Commitment.EVENT_ACTIVATED) {
            commitment.setState(this, Commitment.STATE_ACTIVE);
        } else if (event == Commitment.EVENT_SUCCEEDED) {
            commitment.setState(this, Commitment.STATE_SUCCEEDED);
        } else if (event == Commitment.EVENT_FAILED) {
            commitment.setState(this, Commitment.STATE_FAILED);
        } else if (event == Commitment.EVENT_CHILD_SUCCEEDED) {
            if (commitment.removeCompletedChildren()) {
                commitment.setState(this, Commitment.STATE_SUCCEEDED);
            }
        } else if (event == Commitment.EVENT_CHILD_FAILED) {
            commitment.removeFailedChildren();
            commitment.setState(this, Commitment.STATE_FAILED);
            commitment.notifyChildren(Commitment.EVENT_PARENT_FAILED);
        } else if (event == Commitment.EVENT_PARENT_FAILED) {
            commitment.notifyChildren(Commitment.EVENT_PARENT_FAILED);
            commitment.setState(this, Commitment.STATE_FAILED);
        } else if (event == Commitment.EVENT_REDUNDANT) {
            commitment.notifyChildren(Commitment.EVENT_REDUNDANT);
            commitment.setState(this, Commitment.STATE_REDUNDANT);
        }
        
        super.handleEvent(event);
    }
}
