/*
 * ActionDecoder.java
 *
 * Created on November 5, 2007, 8:36 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.agentfactory.afapl2.interpreter.decoder;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.agentfactory.afapl2.interpreter.mentalState.Action;
import com.agentfactory.logic.MentalAgent;
import com.agentfactory.logic.agent.ActivityIdentifier;
import com.agentfactory.logic.agent.Actuator;
import com.agentfactory.logic.lang.FOS;
import com.agentfactory.logic.lang.LogicFactory;
import com.agentfactory.logic.lang.MalformedLogicException;
import com.agentfactory.logic.lang.TerAND;

/**
 *
 * @author remcollier
 */
public class ActionDecoder implements Decoder {
    private static final String LOGNAME = ActionDecoder.class.getName();
    
    static {
        Logger.getLogger(LOGNAME).setLevel(Level.WARNING);
    }
    
    private String actuator;
    private ActivityIdentifier id;
    private TerAND pre;
    private TerAND post;
    private List<TerAND> postConditions;
    private int cardinality;
    protected List<String> statement;
    protected int index;
    private Map<String, String> configuration;
    private List<String> ontologies;
    
    /** Creates a new instance of ActionDecoder */
    public ActionDecoder(List<String> statement, int offset) {
        this.statement = statement;
        postConditions = new LinkedList<TerAND>();
        ontologies = new ArrayList<String>();
        
        index = offset;
    }
    
    /** Creates a new instance of ActionDecoder */
    public ActionDecoder(List<String> statement) {
        this(statement, 1);
    }
    
    public void decode() throws DecodeException {
        ActivityDecoder activityDecoder = new ActivityDecoder(statement, index);
        activityDecoder.decode();
        id = activityDecoder.getActivityIdentifier();
        index = activityDecoder.getOffset();
        
        BeliefSentenceDecoder bsDecoder = new BeliefSentenceDecoder(statement, index);
        bsDecoder.decode();
        index = bsDecoder.getOffset();
        try {
            Object obj = LogicFactory.create(bsDecoder.getBeliefSentence());
            if (FOS.class.isInstance(obj)) {
                pre = new TerAND();
                pre.add((FOS) obj);
            } else {
                pre = (TerAND) obj;
            }
        } catch (MalformedLogicException mle) {
            Logger.getLogger(LOGNAME).log(Level.SEVERE, "Could not create precondition for: " + id, mle);
        }

        int posts = Integer.parseInt(statement.get(index++));
        while (posts > 0) {
            bsDecoder = new BeliefSentenceDecoder(statement, index);
            bsDecoder.decode();
            index = bsDecoder.getOffset();
            try {
                Object obj = LogicFactory.create(bsDecoder.getBeliefSentence());
                if (FOS.class.isInstance(obj)) {
                    post = new TerAND();
                    post.add((FOS) obj);
                } else {
                    post = (TerAND) obj;
                }
            } catch (MalformedLogicException mle) {
                Logger.getLogger(LOGNAME).log(Level.SEVERE, "Could not create postcondition for: " + id, mle);
            }
            postConditions.add(post);
            posts--;
        }
        
        cardinality = Integer.parseInt(statement.get(index++));
        
        int onts = Integer.parseInt(statement.get(index++));
        while (onts > 0) {
            ontologies.add(statement.get(index++));
            onts--;
        }

        actuator = statement.get(index++);
        
        configuration = new HashMap<String, String>();
        String[] item;
        int items = Integer.parseInt(statement.get(index++));
        while (items > 0) {
            item = statement.get(index++).split("=");
            configuration.put(item[0], item[1]);
            items--;
        }
    }
    
    public int getOffset() {
        return index;
    }
    
    public Action getAction(MentalAgent agent) {
        Action action = null;
        
        try {
            Class actuatorClass = Class.forName(actuator);
            Actuator actu = (Actuator) actuatorClass.newInstance();
            actu.setAgent(agent);
            actu.setConfiguration(configuration);
            actu.setOntologies(ontologies);
            action = new Action(id, pre, postConditions, actu, cardinality);
        } catch (Exception ex) {
            Logger.getLogger(LOGNAME).log(Level.WARNING, "Action could not be instantiated: " + id,ex);
        }
        
        return action;
    }
}
