/*
 * ActionDecoder.java
 *
 * Created on November 5, 2007, 8:36 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.agentfactory.afapl2.interpreter.decoder;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.agentfactory.afapl2.interpreter.mentalState.Plan;
import com.agentfactory.logic.agent.ActivityIdentifier;
import com.agentfactory.logic.lang.FOS;
import com.agentfactory.logic.lang.LogicFactory;
import com.agentfactory.logic.lang.MalformedLogicException;
import com.agentfactory.logic.lang.TerAND;

/**
 *
 * @author remcollier
 */
public class PlanDecoder implements Decoder {
    private static final String LOGNAME = PlanDecoder.class.getName();
    
    static {
        Logger.getLogger(LOGNAME).setLevel(Level.SEVERE);
    }
    
    private FOS plan;
    private ActivityIdentifier id;
    private TerAND pre;
    private TerAND post;
    
    protected List<String> statement;
    protected int index;
    
    /** Creates a new instance of ActionDecoder */
    public PlanDecoder(List<String> statement, int offset) {
        this.statement = statement;
        index = offset;
    }
    
    /** Creates a new instance of ActionDecoder */
    public PlanDecoder(List<String> statement) {
        this(statement, 1);
    }
    
    public void decode() throws DecodeException {
        ActivityDecoder activityDecoder = new ActivityDecoder(statement, index);
        activityDecoder.decode();
        id = activityDecoder.getActivityIdentifier();
        index = activityDecoder.getOffset();
        
        BeliefSentenceDecoder bsDecoder = new BeliefSentenceDecoder(statement, index);
        bsDecoder.decode();
        index = bsDecoder.getOffset();
        try {
            Object obj = LogicFactory.create(bsDecoder.getBeliefSentence());
            if (FOS.class.isInstance(obj)) {
                pre = new TerAND();
                pre.add((FOS) obj);
            } else {
                pre = (TerAND) obj;
            }
        } catch (MalformedLogicException mle) {
            Logger.getLogger(LOGNAME).log(Level.SEVERE, "Could not create postcondition for: " + id, mle);
        }
        
        bsDecoder = new BeliefSentenceDecoder(statement, index);
        bsDecoder.decode();
        index = bsDecoder.getOffset();
        try {
            Object obj = LogicFactory.create(bsDecoder.getBeliefSentence());
            if (FOS.class.isInstance(obj)) {
                post = new TerAND();
                post.add((FOS) obj);
            } else {
                post = (TerAND) obj;
            }
        } catch (MalformedLogicException mle) {
            Logger.getLogger(LOGNAME).log(Level.SEVERE, "Could not create postcondition for: " + id, mle);
        }
        
        FOSDecoder fosDecoder = new FOSDecoder(statement, index);
        fosDecoder.decode();
        plan = new FOS(fosDecoder.getFOS());
        index = fosDecoder.getOffset();
    }
    
    public int getOffset() {
        return index;
    }
    
    public Plan getPlan() {
        return new Plan(id, pre, post, plan);
    }
}
