/*
 * BeliefDecoder.java
 *
 * Created on November 4, 2007, 8:36 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.agentfactory.afapl2.interpreter.decoder;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.agentfactory.afapl2.interpreter.control.RoleTemplate;
import com.agentfactory.logic.lang.FOS;
import com.agentfactory.logic.lang.LogicFactory;
import com.agentfactory.logic.lang.MalformedLogicException;

/**
 *
 * @author remcollier
 */
public class RoleDecoder implements Decoder {
    private static final String LOGNAME = RoleDecoder.class.getName();
    
    static {
        Logger.getLogger(LOGNAME).setLevel(Level.SEVERE);
    }
        
    private List<String> list;
    private int index;
    private RoleTemplate role;
    
    /** Creates a new instance of BeliefDecoder */
    public RoleDecoder(List<String> list) {
        this(list, 1);
    }
    
    /** Creates a new instance of BeliefDecoder */
    public RoleDecoder(List<String> list, int offset) {
        this.list = list;
        index = offset;
    }
    
    public void decode() throws DecodeException {
        FOSDecoder decoder = new FOSDecoder(list, index);
        decoder.decode();
        index = decoder.getOffset();
        role = new RoleTemplate(new FOS(decoder.getFOS()));

        // Role Mental State
        String keyword = null;
        int count = Integer.parseInt(list.get(index++));
        while (count > 0) {
            keyword = list.get(index);
            if (keyword.equals("COM")) {
                CommitmentDecoder cd = new CommitmentDecoder(list, index+1);
                cd.decode();
                index = cd.getOffset();
                role.addCommitment(cd.getCommitment());
            } else {
                System.out.println("[RoleDecoder] Unknown mental state: " + keyword);
            }
            count--;
        }
        
        // Role Triggers
        BeliefSentenceDecoder bsDecoder = null;
        count = Integer.parseInt(list.get(index++));
        while (count > 0) {
            bsDecoder = new BeliefSentenceDecoder(list, index);
            bsDecoder.decode();
            index = bsDecoder.getOffset();
            try {
                role.addTrigger(LogicFactory.create(bsDecoder.getBeliefSentence()));
            } catch (MalformedLogicException mle) {
                Logger.getLogger(LOGNAME).log(Level.SEVERE, "Could not create trigger for role: " + role.getIdentifier(), mle);
            }
            count--;
        }
        
        // Role Rules
        CommitmentRuleDecoder crDecoder = null;
        count = Integer.parseInt(list.get(index++));
        while (count > 0) {
            crDecoder = new CommitmentRuleDecoder(list, index+1);
            crDecoder.decode();
            index = crDecoder.getOffset();
            role.addCommitmentRule(crDecoder.getRule());
            count--;
        }
    }
    
    public RoleTemplate getRoleTemplate() {
        return role;
    }
    
    public int getOffset() {
        return index;
    }
}
