package com.agentfactory.afapl2.interpreter.mentalState;


import java.util.logging.Level;
import java.util.logging.Logger;

import com.agentfactory.logic.agent.ActivityIdentifier;
import com.agentfactory.logic.lang.FOS;
import com.agentfactory.logic.lang.TerAND;

public class Plan implements IActivity {
    private static final String LOGNAME = Plan.class.getName();
    
    static {
        Logger.getLogger(LOGNAME).setLevel(Level.SEVERE);
    }
    
    /**
     * A FOS which is used to identify the plan. The identifier is identicle
     * to the form of an action.
     */
    private ActivityIdentifier identifier;
    
    /**
     * Pre Condition for the plan. This will probably be a set of beliefs
     * which have to be held for the plan to take effect.
     */
    private TerAND preConditions;
    
    /**
     * Post condition for the plan. This will be used to identify what the expected
     * outcome of the plan is. This is a niave point of view since we cannot
     * guarantee that this will be the outcome. This post condition will probably
     * have to identify more than one possible postcondition.
     */
    private TerAND postConditions;
    
    /**
     * Plan Body for the current plan. The plan body is represented by a special
     * plan tree or graph.
     */
    private FOS body;
    
    public Plan(ActivityIdentifier id, TerAND pre, TerAND post, FOS plan) {
        this.identifier = id;
        this.preConditions = pre;
        this.postConditions = post;
        this.body = plan;
    }
    
    /**
     * Return the PlanIdentifier for the current plan. This is the proper
     * FOS version of the plan identifier.
     */
    public ActivityIdentifier getIdentifier() {
        return identifier;
    }
    
    /**
     * Get the plan body associated with a plan
     *
     * @return PlanBody - the body of the current plan.
     */
    public FOS getBody() {
        return body;
    }
    
    /**
     * Get the plan's pre-condition
     */
    public TerAND getPreCondition() {
        return preConditions;
    }
    
    /**
     * Get the plan's post condition
     */
    public TerAND getPostCondition() {
        return postConditions;
    }
    
    
    /**
     * Produce a simple clone of the current plan object. This is used within
     * the Plan class to produced initialised copies of plan templates.
     *
     * @return Object - Actually a Plan object.
     */
    @Override
    public Object clone() {
        throw new RuntimeException("This is the plan clone method,,,");
        /*
        return new Plan((ActivityIdentifier)this.identifier.clone(),
                        (TerAND) preConditions.clone(),
                        (TerAND) postConditions.clone(),
                        (FOS) this.body.clone());
        */
    }
    
    /**
     * Produce a string representation of the plan
     */
    @Override
    public String toString() {
        return "PLAN " + this.identifier + " {" + " PRECONDITIONS; " +
                this.preConditions + "; " + "POSTCONDITIONS " +
                this.postConditions + "; BODY " + this.body + "; }";
    }
}

