package com.agentfactory.agentspeak.interpreter;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agentfactory.agentspeak.interpreter.mentalstate.ASEvent;
import com.agentfactory.clf.interpreter.CoreUtilities;
import com.agentfactory.clf.lang.Predicate;

public class BeliefManagerTest {
	BeliefManager beliefManager = new BeliefManager();
	
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	@Before
	public void setUp() throws Exception {
	}

	@After
	public void tearDown() throws Exception {
	}

	private Predicate addBelief(String belief) {
		Predicate pred = (Predicate) CoreUtilities.factory.createFormula(belief);
		beliefManager.adoptBelief(pred, null);
		return pred;
	}

	private void removeBelief(String belief) {
		Predicate pred = (Predicate) CoreUtilities.factory.createFormula(belief);
		beliefManager.dropBelief(pred, null);
	}

	private void manageBeliefs() {
		beliefManager.manage();
		//beliefManager.generateLostPerceptsEvents();
	}
	private void displayState() {
		System.out.println("Beliefs: " + beliefManager.beliefs().size());
		System.out.println("-----------------");
		for (ASEvent event: beliefManager.events()) {
			System.out.println(event);
		}
		System.out.println("-----------------");
		for (Predicate predicate: beliefManager.beliefs()) {
			System.out.println(CoreUtilities.presenter.toString(predicate));
		}
		System.out.println("-----------------");
	}
	
	@Test
	public void testAddition() {
		beliefManager.copyAndWipePercepts();
		Predicate pred = addBelief("likes(rem, food)[source(self)]");
		manageBeliefs();		
		beliefManager.events();
		if (!beliefManager.beliefs().contains(pred)) Assert.fail("belief not added");
	}

	@Test
	public void testDuplicateAddition() {
		beliefManager.copyAndWipePercepts();
		Predicate pred = addBelief("likes(rem, food)[source(self)]");
		manageBeliefs();
		beliefManager.events();
		beliefManager.copyAndWipePercepts();
		pred = addBelief("likes(rem, food)[source(self)]");
		manageBeliefs();
		beliefManager.events();
		//if (!beliefManager.beliefs().contains(pred)) Assert.fail("belief not added");
	}

	@Test
	public void testSourceAddition() {
		beliefManager.copyAndWipePercepts();
		Predicate pred = addBelief("likes(rem, food)[source(self)]");
		pred = addBelief("likes(rem, food)[source(percept)]");
		manageBeliefs();
		beliefManager.events();
		if (!beliefManager.beliefs().contains(pred)) Assert.fail("belief not added");
	}

	@Test
	public void testEmptyRemoval() {
		beliefManager.copyAndWipePercepts();
		removeBelief("likes(rem, food)[source(self)]");
		manageBeliefs();
		if (!beliefManager.events().isEmpty()) Assert.fail("unexpected event");	}

	@Test
	public void testRemoval() {
		beliefManager.copyAndWipePercepts();
		Predicate pred = addBelief("likes(rem, food)[source(self)]");
		manageBeliefs();
		beliefManager.events();
		beliefManager.copyAndWipePercepts();
		removeBelief("likes(rem, food)[source(self)]");
		manageBeliefs();
		if (beliefManager.events().size() != 1) Assert.fail("incorrect event queue size: " + beliefManager.events().size());
	}
}
