/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package com.agentfactory.astr.debugger;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import com.agentfactory.astr.interpreter.ASEvent;
import com.agentfactory.astr.interpreter.ASTRProgram;
import com.agentfactory.astr.interpreter.AbstractASTRAgent;
import com.agentfactory.astr.interpreter.Intention;
import com.agentfactory.astr.interpreter.PartialPlan;
import com.agentfactory.astr.interpreter.PlanRule;
import com.agentfactory.astr.interpreter.Utilities;
import com.agentfactory.clf.lang.IMPLIES;
import com.agentfactory.clf.lang.Predicate;
import com.agentfactory.visualiser.impl.AbstractLanguageAgentStateManager;
import com.agentfactory.visualiser.impl.DefaultSnapshot;

/**
 *
 * @author remcollier
 */
public class ASTRStateManager extends AbstractLanguageAgentStateManager {
    @Override
    protected DefaultSnapshot createSnapshot() {
        return new ASTRSnapshot();
    }
    
    @Override
    protected void constructSnapshot(DefaultSnapshot shot) {
        super.constructSnapshot(shot);
        
        AbstractASTRAgent agt = (AbstractASTRAgent) agent;
        ASTRSnapshot snap = (ASTRSnapshot) shot;
        
        if (agt.getEvent() != null) {
        	snap.event = agt.getEvent().toString();
        	for (PlanRule rule: agt.getOptions()) {
        		snap.options.add(rule.toString());
        	}
        	if (agt.getSelected() != null) snap.option = agt.getSelected().toString();
        }
        
        for (ASEvent event: agt.getEventQueue()) {
        	snap.events.add(event.toString());
        }
        
        for(Predicate bel: agt.getBeliefs()) {
        	snap.beliefs.add(Utilities.presenter.toString(bel));
        }
        
        for (IMPLIES inference : agt.getInferences()) {
        	snap.inferences.add(Utilities.presenter.toString(inference));
        }
        for (Intention commitment: agt.getCommitments()) {
        	snap.commitments.add(new IntentionDescription(commitment));
        }
        
        for (String action : agt.getActionLog()) {
        	snap.actionLog.add(action);
        }
        
        snap.rules = agt.getRules();
        snap.plans = new LinkedList<PartialPlan>(agt.getPartialPlans().values());
        
		snap.programStack = new HashMap<String, List<String>>(); 
		for (Predicate p : agt.getStack().keySet()) {
			snap.programStack.put(Utilities.presenter.toString(p), agt.getStack().get(p));
		}

		snap.actionTrace = new HashMap<String, List<String>>(); 
		for (Predicate p : agt.getActionTrace().keySet()) {
			snap.actionTrace.put(Utilities.presenter.toString(p), agt.getActionTrace().get(p));
		}
		snap.activePrograms = new LinkedList<String>();
		for (Predicate p : agt.getActivePrograms()) {
			snap.activePrograms.add(Utilities.presenter.toString(p));
		}
		List<String> l = new LinkedList<String>();
		for(ASTRProgram p: agt.getPrograms().values()){
			for(String s: p.getFunction())
			l.add(s);
		}
		snap.functions=l;
    }
}
