package com.agentfactory.astr.interpreter;

import com.agentfactory.clf.lang.IFormula;
import com.agentfactory.clf.lang.Predicate;
import com.agentfactory.clf.lang.Unifier;
import com.agentfactory.clf.reasoner.Bindings;

public class ASEvent {
	public static final int ADDITION 						= 0;
	public static final int REMOVAL							= 1;
	
	private int type;
	private IFormula content;
	private Object source;
	
	public ASEvent(int type, IFormula content) {
		this(type, content, null);
	}
	
	public ASEvent(int type, IFormula content, Object source) {
		this.type = type;
		this.content = content;
		this.source = source;
	}
	
	public IFormula content() {
		return content;
	}
	
	public boolean matches(ASEvent event) {
		return (type == event.type) && (content.matches(event.content));
	}
	
	public boolean isGoalEvent() {
		// Check if the content is an achievement goal...
		return AchievementFormula.class.isInstance(content);
	}
	
	public Object getSource() {
		return source;
	}
	
	public String toString() {
		String output = "+";
		if (type == REMOVAL) output = "-";
		return output + Utilities.presenter.toString(content);
	}
	
	public ASEvent apply(Bindings set) {
		return new ASEvent(type, content.apply(set));
	}
	
	public Bindings unifyWith(ASEvent event) {
		if (event.isGoalEvent() && isGoalEvent()) {
			return Unifier.unifyFormulae((Predicate) ((AchievementFormula) content).formula(), (Predicate) ((AchievementFormula) event.content).formula());
		}
		
		return Unifier.unifyFormulae((Predicate) content, (Predicate) event.content);
	}
	
	public String toSignature() {
		if (isGoalEvent()) {
			return "!" + Utilities.createSignature((Predicate) ((AchievementFormula) content).formula());
		}
		return  Utilities.createSignature((Predicate) content);
	}
		
}
