package com.agentfactory.astr.interpreter;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.clf.lang.IFormula;
import com.agentfactory.clf.lang.Predicate;
import com.agentfactory.clf.plans.IPlanStep;
import com.agentfactory.clf.reasoner.Bindings;

public class PlanRule {
	private ASEvent triggeringEvent;
	private IFormula context;
	private List<IPlanStep> plan;
	private Predicate label;
	private IFormula condition;
	
	public PlanRule(Predicate label, ASEvent event, IFormula context, IFormula condition, List<IPlanStep> plan) {
		this.label = label;
		this.triggeringEvent = event;
		this.context = context;
		this.condition = condition;
		this.plan = plan;
	}
	
	public Predicate getLabel() {
		return label;
	}
	
	public ASEvent getTriggeringEvent() {
		return triggeringEvent;
	}
	
	public IFormula getContext() {
		return context;
	}
	
	public IFormula getCondition() {
		return condition;
	}
	
	public List<IPlanStep> getPlan() {
		return plan;
	}
	
	public boolean isOption(ASEvent event) {
		return triggeringEvent.matches(event);
	}
	
	public String toString() {
		String output = "@" + Utilities.presenter.toString(label) + "\n" +
						triggeringEvent.toString() + " : " +
						Utilities.presenter.toString(context) + " <- [" +
						Utilities.presenter.toString(condition) + "]";
		boolean first = true;
		for (IPlanStep step: plan) {
			if (first) first = false;
			else output += ",";
//			output += "\n" + step.toString(IPlanStep.TAB) + " {" + step.getClass().getName() + "}";
			output += "\n" + step.toString(IPlanStep.TAB);
		}
		return output + ";";
	}
	
	public PlanRule apply(Bindings set) {
		List<IPlanStep> list = new LinkedList<IPlanStep>();
		for (IPlanStep step: plan) {
			list.add(step.apply(set));
		}
		return new PlanRule((Predicate) label.apply(set), triggeringEvent.apply(set), context.apply(set), condition.apply(set), list);
	}
}
