package com.agentfactory.clf.interpreter;

import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.agentfactory.clf.lang.Function;
import com.agentfactory.clf.lang.ITerm;
import com.agentfactory.clf.lang.IntegerTerm;
import com.agentfactory.clf.lang.ListTerm;
import com.agentfactory.clf.lang.Predicate;
import com.agentfactory.clf.lang.RealTerm;
import com.agentfactory.clf.lang.StringTerm;

public abstract class AbstractComponent {
	protected AbstractLanguageAgent agent;
	private Predicate id;
	
	public void setId(Predicate id) {
		this.id = id;
	}
	
	public Predicate getId() {
		return id;
	}
	
	public void setAgent(AbstractLanguageAgent agent) {
		this.agent = agent;
	}
	
	protected void addBelief(String belief) {
		agent.addBelief((Predicate) CoreUtilities.factory.createFormula(belief));
	}
	
	protected void removeBelief(String belief) {
		agent.removeBelief((Predicate) CoreUtilities.factory.createFormula(belief));
	}
	
	public String toString() {
		return id.toString() + "->" + getClass().getName();
	}
	
	protected String termAsString(ITerm _term, int index) {
		ITerm term = _term.terms().get(index);
		if (StringTerm.class.isInstance(term)) {
			return ((StringTerm) term).value();
		}
		return CoreUtilities.presenter.toString(term);
	}
	
	protected String termAsString(Predicate predicate, int index) {
		ITerm term = predicate.termAt(index);
		if (StringTerm.class.isInstance(term)) {
			return ((StringTerm) term).value();
		}
		return CoreUtilities.presenter.toString(term);
	}

	protected int termAsInt(Predicate predicate, int index) {
		ITerm term = predicate.termAt(index);
		if (IntegerTerm.class.isInstance(term)) {
			return ((IntegerTerm) term).value();
		} else if (RealTerm.class.isInstance(term)) {
			return (int) ((RealTerm) term).value();
		}
		return Integer.parseInt(termAsString(predicate, index));
	}

	protected long termAsLong(Predicate predicate, int index) {
		ITerm term = predicate.termAt(index);
		if (IntegerTerm.class.isInstance(term)) {
			return (long) ((IntegerTerm) term).value();
		}
		return Long.parseLong(termAsString(predicate, index));
	}
	
	protected double termAsFloat(Predicate predicate, int index) {
		ITerm term = predicate.termAt(index);
		if (RealTerm.class.isInstance(term)) {
			return (float) ((RealTerm) term).value();
		}
		return Float.parseFloat(termAsString(predicate, index));
	}
	
	protected double termAsDouble(Predicate predicate, int index) {
		ITerm term = predicate.termAt(index);
		if (RealTerm.class.isInstance(term)) {
			return ((RealTerm) term).value();
		}
		return Double.parseDouble(termAsString(predicate, index));
	}
	
	protected Set<String> termAsSet(Predicate predicate, int index) {
		Set<String> set = new HashSet<String>();
		ListTerm list = (ListTerm) predicate.termAt(index);
		for (ITerm term: list.terms()) {
			set.add(CoreUtilities.presenter.toString(term));
		}
		return set;
	}

	protected List<String> termAsList(Predicate predicate, int index) {
		List<String> lst = new LinkedList<String>();
		ListTerm list = (ListTerm) predicate.termAt(index);
		for (ITerm term: list.terms()) {
			lst.add(CoreUtilities.presenter.toString(term));
		}
		return lst;
	}

	protected Map<String, String> termAsMap(Predicate predicate, int index) {
		Map<String, String> map = new HashMap<String, String>();
		ListTerm list = (ListTerm) predicate.termAt(index);
		for (ITerm term: list.terms()) {
			if (Function.class.isInstance(term)) {
				Function funct = (Function) term;
				map.put(funct.functor(), CoreUtilities.presenter.toString(funct.termAt(0)));
			}
		}
		return map;
	}
}