package com.agentfactory.clf.interpreter;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import com.agentfactory.clf.lang.Predicate;

public class Module extends AbstractComponent {
   protected List<Sensor> sensors;
   protected Map<String, Action> actions;

   public Module() {
      sensors = new LinkedList<Sensor>();
      actions = new HashMap<String, Action>();
   }

   @Override
   public void setAgent( AbstractLanguageAgent agent ) {
      super.setAgent( agent );
      for ( Sensor sensor : sensors ) {
         sensor.setAgent( agent );
      }

      for ( Action action : actions.values() ) {
         action.setAgent( agent );
      }
   }

   public void addSensor( String name, Sensor sensor ) {
      sensor.setId( (Predicate) CoreUtilities.factory.createFormula( name ) );
      sensor.setAgent( agent );
      sensor.setModule( this );
      sensors.add( sensor );
   }

   public void addAction( String name, Action action ) {
      action.setId( (Predicate) CoreUtilities.factory.createFormula( name ) );
      action.setAgent( agent );
      action.setModule( this );
      actions.put( CoreUtilities.createSignature( action.getId() ), action );
   }

   public List<Sensor> getSensors() {
      return sensors;
   }

   public Action getAction( String id ) {
      return actions.get( id );
   }

   public Collection<Action> getActions() {
      return actions.values();
   }
}
