package com.agentfactory.clf.interpreter;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

import com.agentfactory.clf.interpreter.Action;
import com.agentfactory.clf.interpreter.CoreUtilities;
import com.agentfactory.clf.interpreter.Module;
import com.agentfactory.clf.interpreter.Sensor;
import com.agentfactory.clf.lang.Predicate;

public class QueueAPI extends Module {
	Map<String, LinkedList<String>> queueMap;
	
	public QueueAPI() {
		queueMap = new HashMap<String, LinkedList<String>>();
		
		addAction("enqueue(?id,?item)", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				String id = CoreUtilities.presenter.toString(activity.termAt(0));
				String item = CoreUtilities.presenter.toString(activity.termAt(1));
				
				LinkedList<String> queue = queueMap.get(id);
				if (queue == null) {
					addBelief("noSuchQueue(" + id +")[source(percept)]");
					return false;
				}
				
				queue.addLast(item);
				return true;
			}
		});
		addAction("create(?id)", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				String id = CoreUtilities.presenter.toString(activity.termAt(0));
				queueMap.put(id, new LinkedList<String>());
				return true;
			}
		});
		addAction("destroy(?id)", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				String id = CoreUtilities.presenter.toString(activity.termAt(0));
				queueMap.remove(id);
				return true;
			}
		});
		addAction("dequeue(?id)", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				String id = CoreUtilities.presenter.toString(activity.termAt(0));
				LinkedList<String> queue = queueMap.get(id);
				if (queue == null) {
					addBelief("noSuchQueue(" + id +")[source(percept)]");
					return false;
				}
				if (queue.isEmpty()) {
					addBelief("queueEmpty(" + id + ")[source(percept)]");
					return false;
				}
				String first = queue.removeFirst();
				addBelief("dequeued(" + id + "," + first + ")[source(percept)]");
				return true;
			}
		});
		addSensor("state", new Sensor() {
			@Override
			public void perceive() {
				for (String name : queueMap.keySet()) {
					addBelief("queue(" + name + ")");
					LinkedList<String> queue = queueMap.get(name);
					if (queue.isEmpty()) {
						addBelief("empty(" + name + ")");
					} else {
						addBelief("front(" + name + ", " + queue.getFirst() + ")");
					}
					addBelief("size(" + name + ", " + queue.size() + ")");
				}
			}
		});
		
	}
}
