package com.agentfactory.clf.interpreter;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

import com.agentfactory.clf.lang.Predicate;

public class StackAPI extends Module {
	Map<String, LinkedList<String>> stackMap;
	
	public StackAPI() {
		stackMap = new HashMap<String, LinkedList<String>>();
		
		addAction("push(?id,?item)", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				String id = CoreUtilities.presenter.toString(activity.termAt(0));
				String item = CoreUtilities.presenter.toString(activity.termAt(1));
				
				LinkedList<String> stack = stackMap.get(id);
				if (stack == null) {
					addBelief("noSuchStack(" + id +")");
					return false;
				}
				
				stack.addFirst(item);
				return true;
			}
		});
		addAction("create(?id)", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				String id = CoreUtilities.presenter.toString(activity.termAt(0));
				stackMap.put(id, new LinkedList<String>());
				return true;
			}
		});
		addAction("destroy(?id)", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				String id = CoreUtilities.presenter.toString(activity.termAt(0));
				stackMap.remove(id);
				return true;
			}
		});
		addAction("pop(?id)", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				String id = CoreUtilities.presenter.toString(activity.termAt(0));
				LinkedList<String> stack = stackMap.get(id);
				if (stack == null) {
					addBelief("noSuchStack(" + id +")");
					return false;
				}
				if (stack.isEmpty()) {
					addBelief("stackEmpty(" + id + ")");
					return false;
				}
				stack.removeFirst();
				return true;
			}
		});
		addSensor("state", new Sensor() {
			@Override
			public void perceive() {
				for (String name : stackMap.keySet()) {
					LinkedList<String> stack = stackMap.get(name);
					if (stack.isEmpty()) {
						addBelief("empty(" + name + ")");
					} else {
						addBelief("top(" + name + ", " + stack.getFirst() + ")");
					}
					addBelief("size(" + name + ", " + stack.size() + ")");
				}
			}
		});
		
	}
}
