package com.agentfactory.clf.lang;

import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import com.agentfactory.clf.reasoner.Bindings;

public class AND implements IFormula,Iterable<IFormula> {
	private List<IFormula> formulae;
	
	public AND(IFormula[] formulae) {
		this.formulae = new LinkedList<IFormula>();
		for (IFormula formula: formulae) {
			this.formulae.add(formula);
		}
	}
	
	public AND(List<IFormula> formulae) {
		this.formulae = formulae;
	}

	/**
	 * Delegate to the first formula to verify type
	 */
	@Override
	public String type() {
		return formulae.get(0).type();
	}
	
	@Override
	public IFormula apply(Bindings set) {
		List<IFormula> list = new LinkedList<IFormula>();
		for (IFormula formula: formulae) {
			list.add(formula.apply(set));
		}
		return new AND(list);
	}

	@Override
	public Iterator<IFormula> iterator() {
		return formulae.iterator();
	}

	@Override
	public Set<Variable> getVariables() {
		Set<Variable> set = new HashSet<Variable>();
		for (IFormula formula: formulae) {
			set.addAll(formula.getVariables());
		}
		return set;
	}
	
	@Override
	public List<IFormula> asList() {
		List<IFormula> list = new LinkedList<IFormula>();
		for (IFormula formula: formulae) {
			list.addAll(formula.asList());
		}
		return list;
	}
	
	public boolean isTrue() {
		for (IFormula formula: formulae) {
			if (formula.isFalse()) return false;
		}
		return true;
	}

	public boolean isFalse() {
	    return !isTrue();
	}

	public boolean matches(IFormula formula) {
		if (AND.class.isInstance(formula)) {
			AND and = (AND) formula;
			for (int i=0; i < formulae.size(); i++) {
				if (!formulae.get(i).matches(and.formulae.get(i))) return false;
			}
			return true;
		}
		return false;
	}
}
