package com.agentfactory.clf.lang;

import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import com.agentfactory.clf.reasoner.Bindings;

public class Bracket implements IFormula {
	private IFormula formula;
	
	public Bracket(IFormula formula) {
		this.formula = formula;
	}
	
	@Override
	public IFormula apply(Bindings set) {
		return new Bracket(formula.apply(set));
	}
	
	public IFormula formula() {
		return formula;
	}
	
	/**
	 * Delegate to the result formula to verify type
	 */
	public String type() {
		return formula.type();
	}
	
	@Override
	public Set<Variable> getVariables() {
		return formula.getVariables();
	}
	
	@Override
	public List<IFormula> asList() {
		List<IFormula> list = new LinkedList<IFormula>();
		list.add(this.formula);
		return list;
	}
	
	public boolean isTrue() {
		return formula.isTrue();
	}

	public boolean isFalse() {
		return formula.isFalse();
	}

	public boolean matches(IFormula formula) {
		if (Bracket.class.isInstance(formula)) {
			return formula.matches(((Bracket) formula).formula);
		}
		return false;
	}
}
