package com.agentfactory.clf.lang;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

public abstract class Comparison implements IFormula {
	protected ITerm left;
	protected ITerm right;
	
	public Comparison(ITerm left, ITerm right) {
		this.left = left;
		this.right = right;
	}

	@Override
	public List<IFormula> asList() {
		List<IFormula> list = new LinkedList<IFormula>();
		list.add(this);
		return list;
	}
	@Override
	public Set<Variable> getVariables() {
		Set<Variable> vars = new HashSet<Variable>();
		vars.addAll(left.getVariables());
		vars.addAll(right.getVariables());
		return vars;
	}

	@Override
	public boolean matches(IFormula formula) {
		if (Comparison.class.isInstance(formula)) {
			Comparison e = (Comparison) formula;
			return left.equals(e.left) && right.equals(e.right);
		}
		return false;
	}
	
	@Override
	public String type() {
		return "raw-logic";
	}
	
	public ITerm left() {
		return left;
	}
	
	public ITerm right() {
		return right;
	}
}
