package com.agentfactory.clf.lang;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import com.agentfactory.clf.reasoner.Bindings;

public class IMPLIES implements IFormula {
	private IFormula condition;
	private IFormula result;
	
	/**
	 * Delegate to the result formula to verify type
	 */
	public String type() {
		return result.type();
	}
	
	public IMPLIES(IFormula condition, IFormula result) {
		this.condition = condition;
		this.result = result;
	}
	
	@Override
	public IFormula apply(Bindings set) {
		return new IMPLIES(condition.apply(set), result.apply(set));
	}

	@Override
	public Set<Variable> getVariables() {
		Set<Variable> set= new HashSet<Variable>();
		set.addAll(condition.getVariables());
		set.addAll(result.getVariables());
		return set;
	}
	
	/**
	 * Only used by QueryEngine to add the lhs of the rule
	 */
	@Override
	public List<IFormula> asList() {
		List<IFormula> list = new LinkedList<IFormula>();
		list.addAll(condition.asList());
		return list;
	}

	public boolean isTrue() {
		return false;
	}

	public boolean isFalse() {
		return false;
	}
	
	public IFormula result() {
		return result;
	}
	
	public IFormula condition() {
		return condition;
	}
	
	public boolean matches(IFormula formula) {
		if (IMPLIES.class.isInstance(formula)) {
			IMPLIES implies = (IMPLIES) formula;
			return condition.matches(implies.condition) && result.matches(implies.result);
		}
		return false;
	}
}
