package com.agentfactory.clf.lang;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import com.agentfactory.clf.reasoner.Bindings;

public class Subtraction implements ITerm {
	private ITerm[] terms;
	
	public Subtraction(ITerm[] terms) {
		this.terms = terms;
	}
	
	@Override
	public ITerm apply(Bindings set) {
		ITerm[] t = new ITerm[2];
		t[0] = terms[0].apply(set);
		t[1] = terms[1].apply(set);
		if (t[0].isConstant() && t[1].isConstant()) {
			if (IntegerTerm.class.isInstance(t[0])) {
				if (IntegerTerm.class.isInstance(t[1])) {
					int answer = ((IntegerTerm) t[0]).value() - ((IntegerTerm) t[1]).value();
					return new IntegerTerm(answer);
				} else if (RealTerm.class.isInstance(t[1])) {
					double answer = ((IntegerTerm) t[0]).value() - ((RealTerm) t[1]).value();
					return new RealTerm(answer);
				}
			} else if (RealTerm.class.isInstance(t[0])) {
				if (IntegerTerm.class.isInstance(t[1])) {
					double answer = ((RealTerm) t[0]).value() - ((IntegerTerm) t[1]).value();
					return new RealTerm(answer);
				} else if (RealTerm.class.isInstance(t[1])) {
					double answer = ((RealTerm) t[0]).value() - ((RealTerm) t[1]).value();
					return new RealTerm(answer);
				}
			}
			throw new NumberFormatException();
		} else if (ListTerm.class.isInstance(t[0]) && ListTerm.class.isInstance(t[1])) {
			List<ITerm> list = new LinkedList<ITerm>();
			
			for (ITerm term: (ListTerm) t[0]) {
				if (!((ListTerm) t[1]).terms().contains(term)) {
					list.add(term.apply(set));
				}
			}
			return new ListTerm(list);
		}
		return new Subtraction(t);
	}

	@Override
	public boolean containsVariable(Variable variable) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean isCompatibleFunction(ITerm term) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean isConstant() {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean isDistinctFunction(ITerm term) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean isNotFunctional(ITerm term) {
		// TODO Auto-generated method stub
		return false;
	}

	public ITerm term(int i) {
		return terms[i];
	}

	@Override
	public boolean hasTerms() {
		// Ignore this property for arithmetic
		// TODO (Rem Collier): Refactor this to be better...
		return false;
	}

	@Override
	public Set<Variable> getVariables() {
		Set<Variable> set= new HashSet<Variable>();
		set.addAll(terms[0].getVariables());
		set.addAll(terms[1].getVariables());
		return set;
	}

	public boolean matches(ITerm term) {
		if (!Subtraction.class.isInstance(term)) {
			return false;
		}
		
		Subtraction op = (Subtraction) term;
		return terms[0].matches(op.terms[0]) && terms[1].matches(op.terms[1]);
	}
	@Override
	public List<ITerm> terms() {
		List<ITerm> terms = new LinkedList<ITerm>();
		terms.add(this.terms[0]);
		terms.add(this.terms[1]);
		return terms;
	}
}
