package com.agentfactory.clf.plans;

import java.util.List;

import com.agentfactory.clf.interpreter.AbstractLanguageAgent;
import com.agentfactory.clf.interpreter.Action;
import com.agentfactory.clf.interpreter.ActiveAction;
import com.agentfactory.clf.interpreter.CoreUtilities;
import com.agentfactory.clf.lang.Predicate;
import com.agentfactory.clf.lang.Variable;
import com.agentfactory.clf.reasoner.Bindings;

public class ActiveActionThread implements Runnable {
	Predicate activity;
	boolean success;
	boolean finished;
	ActiveAction action;
	List<Variable> variables;
	Bindings bindings;
	
	public ActiveActionThread(Predicate activity, ActiveAction action, List<Variable> variables) {
		this.activity = activity;
		this.variables = variables;
		bindings = new Bindings();
		this.action = action;
		finished = false;
	}

	@Override
	public void run() {
		setSuccess(action.execute(activity, variables, bindings));
		setFinished(true);
	}

	public synchronized void setFinished(boolean finished) {
		this.finished = finished;
	}

	public synchronized boolean isFinished() {
		return finished;
	}

	public synchronized void setSuccess(boolean success) {
		this.success = success;
	}

	public synchronized boolean isSuccessful() {
		return success;
	}
	
	public void finish(AbstractLanguageAgent agent, PlanStack planStack) {
		// finished, so check outcome and drop the step
		if (!isSuccessful()) {
			planStack.fail();
		} else {
//			System.out.println("Bindings: " + CoreUtilities.presenter.toString(bindings));
			planStack.updateBindings(bindings);
			agent.logAction(CoreUtilities.presenter.toString(activity));
		}
	}
}
