package com.agentfactory.clf.plans;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.clf.interpreter.AbstractLanguageAgent;
import com.agentfactory.clf.reasoner.Bindings;

public class BlockStep implements IPlanStep {
	private List<IPlanStep> plan;
	int planState;
	
	public BlockStep(List<IPlanStep> plan) {
		this.plan = plan;
		planState = 0;
	}
	
	@Override
	public BlockStep apply(Bindings set) {
		List<IPlanStep> list = new LinkedList<IPlanStep>();
		for (IPlanStep step: plan) {
			list.add(step.apply(set));
		}
		return new BlockStep(list);
	}

	public String toString() {
		return toString("");
	}

	public String toString(String tabs) {
		if (planState == 0) {
			String output = "{";
			boolean first = true;
			for (IPlanStep step: plan) {
				if (first) first = false;
				else output += ",";
				output += "\n" + step.toString(tabs + IPlanStep.TAB);
			}
			return output + "\n" + tabs + "}";
		}
		
		return "";
	}

	public List<IPlanStep> plan() {
		return plan;
	}

	@Override
	public void handle(AbstractLanguageAgent agent, PlanStack planStack) {
		switch (planState) {
		case 0:
			planStack.pushBindings();
			List<IPlanStep> steps = new LinkedList<IPlanStep>();
			for (IPlanStep stp: plan) {
				steps.add(stp);
			}
			planStack.back();
			planStack.addPlan(steps);
			planState = 1;
			break;
		case 1:
			planStack.popBindings();
			planState = 0;
		}
	}
}
