package com.agentfactory.clf.plans;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.clf.interpreter.AbstractLanguageAgent;
import com.agentfactory.clf.interpreter.CoreUtilities;
import com.agentfactory.clf.lang.IFormula;
import com.agentfactory.clf.reasoner.Bindings;
import com.agentfactory.clf.reasoner.QueryResult;

public class ForEachStep implements IPlanStep {
	private IFormula guard;
	private IPlanStep body;
	
	public ForEachStep(IFormula guard, IPlanStep body) {
		this.guard = guard;
		this.body = body;
	}

	@Override
	public IPlanStep apply(Bindings set) {
		return new ForEachStep(guard.apply(set), body.apply(set));
	}

	public String toString() {
		return toString("");
	}
	
	public String toString(String tabs) {
		String output = tabs + "foreach(" + CoreUtilities.presenter.toString(guard) + ")";
		if (!BlockStep.class.isInstance(body)) {
			output += "\n" + IPlanStep.TAB;
		}
		return output + body.toString(tabs);
	}

	public IFormula guard() {
		return guard;
	}
	
	public IPlanStep body() {
		return body;
	}

	@Override
	public void handle(AbstractLanguageAgent agent, PlanStack planStack) {
		planStack.pushBindings();
		QueryResult result = agent.queryAll(guard.apply(planStack.bindings()));
		if (result.found) {
			if (result.bindings.isEmpty()) {
				List<IPlanStep> steps = new LinkedList<IPlanStep>();
				steps.add(body);
				planStack.addPlan(steps);
			} else {
				for (Bindings binding: result.bindings) {
					List<IPlanStep> steps = new LinkedList<IPlanStep>();
					steps.add(body.apply(binding));
					planStack.addPlan(steps);
				}
			}
		}
		planStack.popBindings();
	}
}
