package com.agentfactory.clf.plans;

import java.util.List;

import com.agentfactory.clf.interpreter.AbstractLanguageAgent;
import com.agentfactory.clf.interpreter.Action;
import com.agentfactory.clf.interpreter.ActiveAction;
import com.agentfactory.clf.interpreter.CoreUtilities;
import com.agentfactory.clf.lang.Function;
import com.agentfactory.clf.lang.ITerm;
import com.agentfactory.clf.lang.Predicate;
import com.agentfactory.clf.lang.Variable;
import com.agentfactory.clf.reasoner.Bindings;

public class MultiAssignmentStep implements IPlanStep {
	private List<Variable> variables;
	private ITerm expression;
	private ActiveActionThread thread;
	private boolean durative;
	
	public MultiAssignmentStep(List<Variable> variables, ITerm expression, boolean durative) {
		this.variables = variables;
		this.expression = expression;
		this.durative = durative;
	}
	
	@Override
	public IPlanStep apply(Bindings set) {
		return new MultiAssignmentStep(variables, expression.apply(set), durative);
	}
	
	public ITerm expresion() {
		return expression;
	}

	public String toString() {
		return toString("");
	}
	
	public String toString(String tabs) {
		String output = tabs + "[";
		boolean first = true;
		for (Variable variable : variables) {
			if (first) {
				first = false;
			} else {
				output += ",";
			}
			output += CoreUtilities.presenter.toString(variable);
		}
		output += "] = ";		
		if (durative) output += "durative(";
		output += CoreUtilities.presenter.toString(expression);
		if (durative) output += ")";
		return output;
	}

	@Override
	public void handle(AbstractLanguageAgent agent, PlanStack planStack) {
		// Lazy initialization of action thread (performed the first time
		// the step is handled)
		if (thread == null) {
			ITerm term = expression.apply(planStack.bindings());
			if (Function.class.isInstance(term)) {
				Predicate activity = CoreUtilities.factory.convertToPredicate(term);

				// Check the action type here...
				Action action = agent.getActionMap().get(CoreUtilities.createSignature(activity));
				if (action == null) {
					action = agent.getModuleLevelAction(activity);
					if (action == null) {
						// No valid action, so treat the activity as a term...
						// then quit the plan step
						for (Variable variable : variables) {
							planStack.updateBinding(variable, term);
						}
						return;
					}
				}
					
				if (!ActiveAction.class.isInstance(action)) {
					System.err.println("[" + agent.getName() + "] Attempted to activate non-active action: " + CoreUtilities.presenter.toString(activity));
					for (Variable variable : variables) {
						planStack.updateBinding(variable, term);
					}
				}
				
				thread = new ActiveActionThread(activity, (ActiveAction) action, variables);
				if (durative) {
					new Thread(thread).start();
				} else {
					thread.run();
				}
			} else {
				for (Variable variable : variables) {
					planStack.updateBinding(variable, term);
				}
			}
		}
		
		// each time the step is handled, check if the action has finished
		if (thread.isFinished()) {
			thread.finish(agent, planStack);
			thread = null;
		} else {
			// not finished, so put the step back on the plan stack.
			planStack.back();

		}			
	}
}
