package com.agentfactory.clf.plans;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.clf.interpreter.AbstractLanguageAgent;
import com.agentfactory.clf.interpreter.CoreUtilities;
import com.agentfactory.clf.lang.AND;
import com.agentfactory.clf.lang.ITerm;
import com.agentfactory.clf.lang.ListTerm;
import com.agentfactory.clf.lang.Variable;
import com.agentfactory.clf.reasoner.Bindings;
import com.agentfactory.clf.reasoner.QueryResult;

public class NewListStep implements IPlanStep {
	private ITerm term;
	private AND context;
	private ITerm variable;
	
	public NewListStep(ITerm term, AND context, ITerm variable) {
		this.term = term;
		this.context = context;
		this.variable = variable;
	}
	
	@Override
	public IPlanStep apply(Bindings set) {
		return new NewListStep(term.apply(set), (AND) context.apply(set), variable);
	}

	public String toString() {
		return toString("");
	}
	
	public String toString(String tabs) {
		return tabs + CoreUtilities.presenter.toString(variable) + " = list [" + 
				CoreUtilities.presenter.toString(term) + 
				" | " + CoreUtilities.presenter.toString(context) + "]";
	}

	public ITerm variable() {
		return variable;
	}
	
	public ITerm term() {
		return term;
	}
	
	public AND context() {
		return context;
	}

	@Override
	public void handle(AbstractLanguageAgent agent, PlanStack planStack) {
		QueryResult results = agent.queryAll(context);
		if (results.found) {
			List<ITerm> terms = new LinkedList<ITerm>();
			
			for (Bindings binding : results.bindings) {
				terms.add(term.apply(binding));
			}
			
			ListTerm list = new ListTerm(terms);
			planStack.updateBinding((Variable) variable, list);
		} else {
			planStack.fail();
		}
	}
}
