package com.agentfactory.clf.plans;

import com.agentfactory.clf.interpreter.AbstractLanguageAgent;
import com.agentfactory.clf.interpreter.CoreUtilities;
import com.agentfactory.clf.lang.IFormula;
import com.agentfactory.clf.reasoner.Bindings;
import com.agentfactory.clf.reasoner.QueryResult;

public class QueryStep implements IPlanStep {
	protected IFormula step;

	public QueryStep(IFormula step) {
		this.step = step;
	}
	
	@Override
	public IPlanStep apply(Bindings set) {
		return new QueryStep(step.apply(set));
	}
	
	public IFormula formula() {
		return step;
	}

	public String toString() {
		return toString("");
	}
	
	public String toString(String tabs) {
		return tabs + "query(" + CoreUtilities.presenter.toString(step) + ")";
	}

	@Override
	public void handle(AbstractLanguageAgent agent, PlanStack planStack) {
		QueryResult result = agent.queryAll(step.apply(planStack.bindings()));
		if (!result.found) {
			planStack.fail();
		} else {
			if (!result.bindings.isEmpty()) {
				planStack.updateBindings(result.bindings.get(0));
			}
		}
	}
}
