package com.agentfactory.clf.plans;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.clf.interpreter.AbstractLanguageAgent;
import com.agentfactory.clf.interpreter.CoreUtilities;
import com.agentfactory.clf.lang.IFormula;
import com.agentfactory.clf.reasoner.Bindings;
import com.agentfactory.clf.reasoner.QueryResult;

public class WhenStep implements IPlanStep {
	private IFormula guard;
	private IPlanStep plan;
	
	public WhenStep(IFormula guard, IPlanStep plan) {
		this.guard = guard;
		this.plan = plan;
	}
	@Override
	public IPlanStep apply(Bindings set) {
		return new WhenStep(guard.apply(set), plan.apply(set));
	}

	public String toString() {
		return toString("");
	}
	
	public String toString(String tabs) {
		String output = tabs + "when (" + CoreUtilities.presenter.toString(guard) + ") ";
		if (!BlockStep.class.isInstance(plan)) {
			output += plan.toString();
		} else {
			output += plan.toString(tabs+IPlanStep.TAB);
		}
		
		return output;
	}

	public IFormula guard() {
		return guard;
	}
	
	public IPlanStep plan() {
		return plan;
	}
	
	@Override
	public void handle(AbstractLanguageAgent agent, PlanStack planStack) {
		QueryResult result = agent.queryAll(guard.apply(planStack.bindings()));
		if (result.found) {
			List<IPlanStep> steps = new LinkedList<IPlanStep>();
			if (result.bindings.isEmpty()) {
				steps.add(plan);
			} else {
				steps.add(plan.apply(result.bindings.get(0)));
			}
			planStack.addPlan(steps);
		} else {
			planStack.back();
		}
	}
}
