package com.agentfactory.clf.test;

import static org.junit.Assert.assertEquals;

import java.util.LinkedList;
import java.util.List;

import org.junit.Before;
import org.junit.Test;

import com.agentfactory.clf.interpreter.CoreUtilities;
import com.agentfactory.clf.lang.IFormula;
import com.agentfactory.clf.lang.ITerm;
import com.agentfactory.clf.lang.Predicate;
import com.agentfactory.clf.reasoner.Bindings;
import com.agentfactory.clf.reasoner.IQueryEngine;
import com.agentfactory.clf.reasoner.IQueryable;
import com.agentfactory.clf.reasoner.QueryResult;
import com.agentfactory.clf.reasoner.ResolutionBasedQueryEngine;
import com.agentfactory.clf.util.DefaultLogicFactory;
import com.agentfactory.clf.util.DefaultLogicPresenter;

public class ResolutionReasonerTest {
	static DefaultLogicFactory factory = new DefaultLogicFactory();
	static DefaultLogicPresenter presenter = new DefaultLogicPresenter();

	TestSource source;
	IQueryEngine engine;
	
    @Before
    public void setup() {
		engine = new ResolutionBasedQueryEngine();
		source = new TestSource();
		source.add(factory.createFormula("location(0.0, 9.0)"));
		source.add(factory.createFormula("goto(1,2)"));
		source.add(factory.createFormula("message(inform, agentID(Boss,addr),goto(1,1))"));
		source.add(factory.createFormula("goto(?x,?y) & message(inform, agentID(Boss,?addr),goto(?z,?w))& ~((?x == ?z) & (?y == ?w)) => remove(goto(?x,?y))"));
		source.add(factory.createFormula("isTower([c])"));
		source.add(factory.createFormula("isTower([d])"));
		source.add(factory.createFormula("isTower([e])"));
		engine.addSource(source);
	}
	
	@Test
	public void testStringComp() {
		IFormula p1 = factory.createFormula("test(\"rem\" + \"is\" + \"cool\")");
//		IFormula p1 = factory.createFormula("test(\"rem\" + \"is\")");
		CoreUtilities.displayAST((Predicate) p1);
		p1 = p1.apply(new Bindings());
		CoreUtilities.displayAST((Predicate) p1);
		System.out.println(CoreUtilities.presenter.toString(p1));
	}
	
	@Test
	public void testQuery() {
		IFormula p1 = factory.createFormula("~location(0.0, 0.0)");
		System.out.println("formula: " + presenter.toString(p1));
		QueryResult result = engine.query(p1);
		assertEquals(true, result.found);
	}

	@Test
	public void testBracketQuery() {
		IFormula p1 = factory.createFormula("remove(goto(?x,?y))");
		System.out.println("formula: " + presenter.toString(p1));
		QueryResult result = engine.query(p1);
		assertEquals(true, result.found);
	}

	@Test
	public void testInequality() {
		IFormula p1 = factory.createFormula("rem != fred");
		IFormula p2 = factory.createFormula("rem != rem");
		System.out.println("formula: " + presenter.toString(p1));
		System.out.println("formula: " + presenter.toString(p2));
		QueryResult result = engine.query(p1);
		assertEquals(true, result.found);
		result = engine.query(p2);
		assertEquals(true, !result.found);
	}
	
	@Test
	public void containsTest() {
		IFormula p1 = factory.createFormula("$([a,b,c],c)");
		System.out.println("contains formula: " + presenter.toString(p1));
		QueryResult result = engine.query(p1);
		assertEquals(true, result.found);
	}
	private class TestSource implements IQueryable {
		List<IFormula> list = new LinkedList<IFormula>();

		public void add(IFormula formula) {
			list.add(formula);
		}
		
		@Override
		public boolean hasType(String type) {
			return true;
		}

		@Override
		public List<IFormula> query(IFormula query) {
			return list;
		}
	}
}
