package com.agentfactory.eis;

import java.net.URL;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import com.agentfactory.afapl2.debugger.AFAPL2StateManagerFactory;
import com.agentfactory.afapl2.debugger.inspector.AFAPL2InspectorFactory;
import com.agentfactory.afapl2.interpreter.AFAPL2ArchitectureFactory;
import com.agentfactory.agentspeak.debugger.AgentSpeakStateManagerFactory;
import com.agentfactory.agentspeak.debugger.inspector.AgentSpeakInspectorFactory;
import com.agentfactory.agentspeak.interpreter.AgentSpeakArchitectureFactory;
import com.agentfactory.platform.core.ConfigurationException;
import com.agentfactory.teleoreactive.debugger.TeleoReactiveStateManagerFactory;
import com.agentfactory.teleoreactive.debugger.inspector.TeleoReactiveInspectorFactory;
import com.agentfactory.teleoreactive.interpreter.TeleoReactiveArchitectureFactory;
import com.agentfactory.visualiser.DebuggerRunConfiguration;

public class EISDebugConfiguration extends DebuggerRunConfiguration {
	private String jar, name;
	Map<String, String> designs;
	List<String> beliefs;
	
	public EISDebugConfiguration(String name, Map<String, String> designs, List<String> beliefs, String jar) {
		this.name = name;
		this.jar = jar;
		this.designs = designs;
		this.beliefs = beliefs;
	}
	
	public EISDebugConfiguration(String name, Map<String, String> designs, String jar) {
		this(name, designs, new LinkedList<String>(), jar);
	}
	
	public String getName() {
		return name;
	}
	
	public String getDomain() {
		return "eis.ucd.ie";
	}
	
	public void configure() throws ConfigurationException {
		addLanguageSupport(new AFAPL2InspectorFactory(), new AFAPL2StateManagerFactory());
		addLanguageSupport(new AgentSpeakInspectorFactory(), new AgentSpeakStateManagerFactory());
		addLanguageSupport(new TeleoReactiveInspectorFactory(), new TeleoReactiveStateManagerFactory());
		
		super.configure();
		
		addArchitectureFactory(new AFAPL2ArchitectureFactory());
		addArchitectureFactory(new AgentSpeakArchitectureFactory());
		addArchitectureFactory(new TeleoReactiveArchitectureFactory());
	
		addPlatformService(EISService.class, "eis");
		
        addAgent("manager", "com/agentfactory/eis/EnvironmentManager.aspeak");
        initAgent("manager", "environment(eis, \"" + jar + "\")");
        
        for (String belief: beliefs) {
        	initAgent("manager", belief);
        }
        for (String design: designs.keySet()) {
        	initAgent("manager", "design("+design+", \"" + designs.get(design) + "\")");
        }
	}
}
