package com.agentfactory.logic.agent.beliefManagement;

import com.agentfactory.logic.MentalAgent;
import com.agentfactory.logic.lang.FOS;
import com.agentfactory.logic.lang.TerImplication;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * This belief manager implements the traditional Agent Factory belief
 * management model. It incorporates perception based belief (via the
 * PerceptionBasedBeliefManager) class; basic and temporal beliefs (via
 * the TemporalBeliefStore); and belief rules (via the BeliefRuleStore).
 * beliefs
 * @author rem
 */
public class PersistentBeliefManager extends PerceptionBasedBeliefManager implements IBeliefManagerRuleSupport {
    protected Map<String, BeliefStore> beliefStoreMap;
    protected BeliefRuleStore beliefRules;

    public PersistentBeliefManager(MentalAgent agent) {
        super(agent);

        beliefStoreMap = new HashMap<String, BeliefStore>();
        beliefStoreMap.put("main", new BeliefStore("main"));
        beliefRules = new BeliefRuleStore();
    }

    @Override
    public void adopt(FOS belief) {
        beliefStoreMap.get("main").addBelief(belief);
    }

    @Override
    public void drop(FOS belief) {
        beliefStoreMap.get("main").removeBelief(belief);
    }

    @Override
    public List<Object> query(FOS query) {
        List<Object> list = super.query(query);
        for (BeliefStore store: beliefStoreMap.values()) {
            list.addAll(store.query(query));
        }
        list.addAll(beliefRules.query(query));
        return list;
    }

    @Override
    public List<FOS> getBeliefs() {
        List<FOS> list = new LinkedList<FOS>();
        for (BeliefStore store: beliefStoreMap.values()) {
            list.addAll(store.getBeliefs());
        }
        for (BeliefStore store: perceptions.values()) {
            list.addAll(store.getBeliefs());
        }
        return list;
    }

    public boolean hasType(String type) {
        return type.equals("BELIEF");
    }

    public void addRule(TerImplication rule) {
        beliefRules.addRule(rule);
    }

    public List<TerImplication> getRules() {
        return beliefRules.getRules();
    }
}
