/**
 * Title:       Substitution.java
 * Copyright:   Copyright (c) 1996-2004 The Agent Factory Working Group. All rights reserved.
 * Licence:     This file is free software; you can redistribute it and/or modify
 *              it under the terms of the GNU Lesser General Public License as published by
 *              the Free Software Foundation; either version 2.1, or (at your option)
 *              any later version.
 *
 *              This file is distributed in the hope that it will be useful,
 *              but WITHOUT ANY WARRANTY; without even the implied warranty of
 *              MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *              GNU Lesser General Public License for more details.
 *
 *              You should have received a copy of the GNU Lesser General Public License
 *              along with Agent Factory; see the file COPYING.  If not, write to
 *              the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *              Boston, MA 02111-1307, USA.
 */

package com.agentfactory.logic.reasoner;

import com.agentfactory.logic.lang.FOS;


/**
 *
 * @author Rem Collier
 * @version July 2002
 */
public class Substitution implements Cloneable {
    private FOS first, second;
    
    public Substitution(FOS first, FOS second) {
        this.first = first;
        this.second = second;
    }
    
    public FOS getFirst() {
        return first;
    }
    
    public FOS getSecond() {
        return second;
    }
    
    public boolean isSameAs(Substitution s) {
        return toString().equals(s.toString());
    }
    
    @Override
    public String toString() {
        return first.toString() + " / " +second.toString();
    }
    
    public Substitution apply(SubstitutionSet sub) {
        return new Substitution(first.apply(sub), second.apply(sub));
    }
    
    public Substitution inverse() {
        return new Substitution(second, first);
    }
    
    @Override
    public boolean equals(Object o) {
        Substitution subs = null;
        if (o instanceof Substitution) {
            subs = (Substitution) o;
            if (first.equals(subs.first) && second.equals(subs.second)) {
                return true;
            }
        }
        
        return false;
    }
    
    @Override
    public Object clone() {
        return new Substitution((FOS) first.clone(), (FOS) second.clone());
    }
}

