package com.agentfactory.logic.update.lang;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import com.agentfactory.logic.update.interpreter.CoreUtilities;
import com.agentfactory.logic.update.reasoner.Bindings;

public class Merge implements ITerm {
	private ITerm[] terms;
	
	public Merge(ITerm[] terms) {
		this.terms = terms;
	}
	
	@Override
	public ITerm apply(Bindings set) {
//		System.out.println(CoreUtilities.presenter.toString(set));
		ITerm[] temp = new ITerm[] {terms[0].apply(set), terms[1].apply(set)};
//		System.out.println(terms[0].toString() + ": '"+ CoreUtilities.presenter.toString(terms[0]) +"'");
//		System.out.println(terms[1].toString() + ": '"+ CoreUtilities.presenter.toString(terms[1])+"'");

		if (ListTerm.class.isInstance(temp[0]) && ListTerm.class.isInstance(temp[1])) {
			List<ITerm> list = new LinkedList<ITerm>();
			
			for (ITerm term: (ListTerm) temp[0]) {
				list.add(term.apply(set));
			}
			for (ITerm term: (ListTerm) temp[1]) {
				list.add(term.apply(set));
			}
			
			return new ListTerm(list);
		}
		return new Merge(temp);
	}

	@Override
	public boolean containsVariable(Variable variable) {
		return terms[0].containsVariable(variable) || terms[1].containsVariable(variable);
	}

	@Override
	public boolean isCompatibleFunction(ITerm term) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean isConstant() {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean isDistinctFunction(ITerm term) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean isNotFunctional(ITerm term) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean hasTerms() {
		// Ignore this property for arithmetic
		// TODO (Rem Collier): Refactor this to be better...
		return false;
	}

	@Override
	public Set<Variable> getVariables() {
		Set<Variable> set = new HashSet<Variable>();
		set.addAll(terms[0].getVariables());
		set.addAll(terms[1].getVariables());
		return set;
	}

	public boolean matches(ITerm term) {
		if (!Merge.class.isInstance(term)) {
			return false;
		}
		
		Merge op = (Merge) term;
		return terms[0].matches(op.terms[0]) && terms[1].matches(op.terms[1]);
	}

	public ITerm term(int i) {
		return terms[i];
	}
}
