package com.agentfactory.logic.update.plans;

import com.agentfactory.logic.update.interpreter.AbstractLanguageAgent;
import com.agentfactory.logic.update.interpreter.CoreUtilities;
import com.agentfactory.logic.update.lang.Function;
import com.agentfactory.logic.update.lang.ITerm;
import com.agentfactory.logic.update.lang.IntegerTerm;
import com.agentfactory.logic.update.lang.RealTerm;
import com.agentfactory.logic.update.lang.Variable;
import com.agentfactory.logic.update.reasoner.Bindings;

public class AssignmentStep implements IPlanStep {
	private ITerm variable;
	private ITerm expression;
	
	public AssignmentStep(ITerm variable, ITerm expression) {
		this.variable = variable;
		this.expression = expression;
	}
	
	@Override
	public IPlanStep apply(Bindings set) {
		return new AssignmentStep(variable, expression.apply(set));
	}

	public String toString() {
		return CoreUtilities.presenter.toString(variable) + "=" + CoreUtilities.presenter.toString(expression);
	}

	public ITerm variable() {
		return variable;
	}
	
	public ITerm expression() {
		return expression;
	}

	@Override
	public void handle(AbstractLanguageAgent agent, PlanStack planStack) {
		ITerm expr = expression.apply(planStack.bindings());
		if (Variable.class.isInstance(variable) && isValidTerm(expr)) {
			planStack.updateBinding((Variable) variable, expr);
		} else {
			//commitment.setState(FAILED);
		}
	}
	
	private boolean isValidTerm(ITerm term) {
		return Function.class.isInstance(term) || IntegerTerm.class.isInstance(term) || RealTerm.class.isInstance(term); 
	}
}
