package com.agentfactory.logic.update.plans;

import com.agentfactory.logic.update.interpreter.AbstractLanguageAgent;
import com.agentfactory.logic.update.interpreter.Action;
import com.agentfactory.logic.update.interpreter.CoreUtilities;
import com.agentfactory.logic.update.interpreter.Module;
import com.agentfactory.logic.update.lang.ITerm;
import com.agentfactory.logic.update.lang.Predicate;
import com.agentfactory.logic.update.reasoner.Bindings;

public class DoStep implements IPlanStep {
	private ITerm module;
	private Predicate action;
	
	public DoStep(ITerm module, Predicate action) {
		this.module = module;
		this.action = action;
	}
	
	@Override
	public IPlanStep apply(Bindings set) {
		return new DoStep(module.apply(set), (Predicate) action.apply(set));
	}

	public String toString() {
		return "do (" + CoreUtilities.presenter.toString(module) + ", " + CoreUtilities.presenter.toString(action) + ")";
	}

	public ITerm module() {
		return module;
	}
	
	public Predicate action() {
		return action;
	}

	@Override
	public void handle(AbstractLanguageAgent agent, PlanStack planStack) {
		Module mod = agent.getModule(CoreUtilities.factory.convertToPredicate(module));
		if (mod == null) {
			noSuchModule(agent);
			return;
		}
		Action a = mod.getAction(CoreUtilities.createSignature(action));
		if (a == null) {
			noSuchAction(agent);
			return;
		}
		if (!a.execute(action)) {
			// TODO: Commitment Failure
		}
	}
	
	private void noSuchModule(AbstractLanguageAgent agent) {
		String mod = CoreUtilities.presenter.toString(module);
		Predicate predicate = (Predicate) CoreUtilities.factory.createFormula("unknownModule(" + mod + ")[source(percept)]");
		agent.addBelief(predicate);
	}

	private void noSuchAction(AbstractLanguageAgent agent) {
		String mod = CoreUtilities.presenter.toString(module);
		String act = CoreUtilities.presenter.toString(action);
		Predicate predicate = (Predicate) CoreUtilities.factory.createFormula("unknownModuleAction(" + mod + "," + act +")[source(percept)]");
		agent.addBelief(predicate);
	}
}
