package com.agentfactory.logic.update.plans;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.logic.update.interpreter.AbstractLanguageAgent;
import com.agentfactory.logic.update.interpreter.CoreUtilities;
import com.agentfactory.logic.update.lang.IFormula;
import com.agentfactory.logic.update.reasoner.Bindings;
import com.agentfactory.logic.update.reasoner.QueryResult;

public class WhileStep implements IPlanStep {
	private IFormula guard;
	private List<IPlanStep> plan;
	
	public WhileStep(IFormula guard, List<IPlanStep> plan) {
		this.guard = guard;
		this.plan = plan;
	}
	@Override
	public IPlanStep apply(Bindings set) {
		List<IPlanStep> list = new LinkedList<IPlanStep>();
		for (IPlanStep step: plan) {
			list.add(step.apply(set));
		}
		return new WhileStep(guard.apply(set), list);
	}

	public String toString() {
		String output = "while (" + CoreUtilities.presenter.toString(guard) + ") {";
		boolean first = true;
		for (IPlanStep step: plan) {
			if (first) first = false;
			else output += ",";
			output += step.toString();
		}
		return output + "}";
	}

	public IFormula guard() {
		return guard;
	}
	
	public List<IPlanStep> plan() {
		return plan;
	}
	@Override
	public void handle(AbstractLanguageAgent agent, PlanStack planStack) {
		QueryResult result = agent.queryAll(guard.apply(planStack.bindings()));
		if (result.found) {
			planStack.back();
			List<IPlanStep> steps = new LinkedList<IPlanStep>();
			for (IPlanStep stp: plan) {
				steps.add(stp);
			}
			planStack.addPlan(steps);
		}
	}
}
