package com.agentfactory.platform.impl;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Observable;

import com.agentfactory.platform.core.DuplicateAgentNameException;
import com.agentfactory.platform.core.IAgent;
import com.agentfactory.platform.core.IAgentContainer;
import com.agentfactory.platform.core.NoSuchAgentException;

/**
 * This is the standard basic implementation of the IAgent container.
 * 
 * @author remcollier
 */
public abstract class AbstractAgentContainer extends Observable implements IAgentContainer {
    protected List<IAgent> agents;
    private Map<String, IAgent> agentMap;

    public AbstractAgentContainer() {
        agents = new LinkedList<IAgent>();
        agentMap = new HashMap<String, IAgent>();
    }

    public IAgent getAgentByName( String name ) throws NoSuchAgentException {
        if ( !this.agentMap.containsKey( name ) ) {
            throw new NoSuchAgentException();
        }
        return this.agentMap.get( name );
    }

    public void add( IAgent agent ) throws DuplicateAgentNameException {
        if ( agentMap.get( agent.getName() ) != null ) {
            throw new DuplicateAgentNameException();
        }

        agent.setContainer( this );
        agents.add( agent );
        agentMap.put( agent.getName(), agent );

        setChanged();
        notifyObservers( "created " + agent.getName() );
    }

    public void remove( String name ) throws NoSuchAgentException {
        IAgent agent = agentMap.get( name );

        if ( agent == null ) {
            throw new NoSuchAgentException();
        }

        agents.remove( agent );
        agentMap.remove( name );

        setChanged();
        notifyObservers( "terminated " + name );
    }

    public IAgent get( String name ) throws NoSuchAgentException {
        IAgent agent = agentMap.get( name );
        if ( agent == null ) {
            throw new NoSuchAgentException();
        }

        return agent;
    }

    public IAgent[] agents() {
        return (IAgent[]) agents.toArray( new IAgent[ 1 ] );
    }
}
