/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package com.agentfactory.platform.impl;


import com.agentfactory.platform.core.AgentCreationException;
import com.agentfactory.platform.core.IAgent;
import com.agentfactory.platform.core.IArchitectureFactory;
import com.agentfactory.platform.core.IArchitectureService;
import com.agentfactory.platform.core.NoSuchArchitectureException;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author remcollier
 */
public class ArchitectureService implements IArchitectureService {
    List<IArchitectureFactory> architectures;
    
    public ArchitectureService() {
        architectures = new ArrayList<IArchitectureFactory>();
        registerArchitectureFactory(new JavaArchitectureFactory());
    }
    
    public void registerArchitectureFactory(IArchitectureFactory factory) {
        architectures.add(factory);
    }
    
    /**
     * This method locates the relevant architecture factory and, if one exists,
     * invokes it to create the corresponding agent.
     * 
     * @param name the name of the agent
     * @param design an object containing the design of the agent
     * @return an agent instance of the type corresponding to the design.
     * @throws AgentCreationException 
     * 
     * @throws com.agentfactory.platform.core.NoSuchAgentDesignException
     */
    public IAgent createAgent(String name, Object design) throws NoSuchArchitectureException, AgentCreationException {
        IAgent agent = null;
        
        int index = 0;
        while ((agent == null) && (index < architectures.size())) {
        	IArchitectureFactory factory = architectures.get(index++);
            
            if (factory.filter(design)) {
                agent = factory.create(name, design);
            }
        }
        
        if (agent == null) {
            throw new NoSuchArchitectureException("No Architecture Factory for: " + design);
        }
        
        return agent;
    }
}
