package com.agentfactory.service.df;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.agentfactory.platform.core.AgentID;

public class ServiceDescription {
	public String name;
	public String type;
	public Set<String> protocols;
	public Set<String> ontologies;
	public Set<String> languages;
	public AgentID ownership;
	public Map<String, String> properties;
	
	public ServiceDescription(String name, String type, AgentID agentID) {
		this(name, type, new HashSet<String>(), new HashSet<String>(), new HashSet<String>(), agentID, new HashMap<String,String>());
	}
	
	/**
	 * Create a new service description
	 * @param name
	 * @param type
	 * @param protocols
	 * @param ontologies
	 * @param languages
	 * @param ownership
	 * @param properties
	 */
	public ServiceDescription(String name,
							String type, 
							Set<String> protocols,
							Set<String> ontologies,
							Set<String> languages,
							AgentID ownership,
							Map<String, String> properties) {
		this.name = name;
		this.type = type;
		this.protocols = protocols;
		this.ontologies = ontologies;
		this.languages = languages;
		this.ownership = ownership;
		this.properties = properties;
	}
	
	public void update(ServiceDescription description) {
		type = description.type;
		protocols = description.protocols;
		ontologies = description.ontologies;
		languages = description.languages;
		ownership = description.ownership;
		properties = description.properties;
	}
	
	public boolean matches(ServiceDescription description) {
		if (!name.matches(description.name)) return false;
		if (!type.matches(description.type)) return false;
		if (!Utilities.matcher(protocols, description.protocols)) return false;
		if (!Utilities.matcher(ontologies, description.ontologies)) return false;
		if (!Utilities.matcher(languages, description.languages)) return false;
		if (!ownership.matches(description.ownership)) return false;
		
		// Property names should be exact matches...
		for (String name: description.properties.keySet()) {
			if (!properties.containsKey(name)) return false;
			if (!properties.get(name).matches(description.properties.get(name))) return false;
		}
		
		return true;
	}
}
