package com.agentfactory.service.migration;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;

public class MigrationServer implements Runnable {
	MigrationService service;
	int port;
	ServerSocket server;
	
	public MigrationServer(int port, MigrationService service) {
		this.port = port;
		this.service = service;
	}
	
	@Override
	public void run() {
		try {
			server = new ServerSocket(port);
		} catch (IOException e) {
			System.out.println("[Migration Service] Could not create migration server: " + e.getMessage());
			return;
		}

		
		Socket socket = null;
		try {
			while ((socket = server.accept()) != null) {
				new Thread(new Handler(socket)).start();
			}
		} catch (IOException e) {
			System.out.println("[Migration Service] Catestrophic failure when accepting a connection from a remote platform: " + e.getMessage());
			return;
		}
		
	}

	private class Handler implements Runnable {
		Socket socket;
		
		public Handler(Socket socket) {
			this.socket = socket;
		}

		public void run() {
			try {
				ObjectInputStream in = null;
				ObjectOutputStream out = null;
				
				in = new ObjectInputStream(socket.getInputStream());
				out = new ObjectOutputStream(socket.getOutputStream());
				
				IMigrationSnapshot snapshot = (IMigrationSnapshot) in.readObject();
				out.writeBoolean(service.receiveAgent(snapshot));
				out.flush();
				if (in.readObject().equals("DONE")) {
					service.startAgent(snapshot.getName());
				}
				in.close();
				out.close();
				socket.close();
			} catch (Exception e) {
				System.out.println("[Migration Service] Failed to handle incoming connection from a remote platform: " + e.getMessage());
				e.printStackTrace();
				return;
			}
		}
	}
}
