package com.agentfactory.service.mts.http;

import com.agentfactory.platform.core.Constants;
import com.agentfactory.platform.core.IEnvelope;
import com.agentfactory.platform.core.IMessage;
import com.agentfactory.platform.core.IPlatformServiceManager;
import com.agentfactory.platform.impl.MessageTransportService;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;
import java.util.StringTokenizer;

/**
 * This class is responsible for handling individual requests from the
 * HTTPServer class, parsing the request, checking it for validity,
 * and returning the appropriate response.
 *
 * @author  Rem Collier
 */
public class HTTPRequestHandler extends Thread {
    private String body;
    private String boundary;
    private Socket client;
    private MessageTransportService service;
    //private IPlatformServiceManager manager;
    
    /** Creates a new instance of RequestHandler */
    public HTTPRequestHandler(Socket client, IPlatformServiceManager manager,
                              MessageTransportService service) {
        this.client = client;
        //this.manager = manager;
        this.service = service;
    }
    
    public void run() {
        InputStreamReader reader = null;
        BufferedOutputStream outputStream = null;
        BufferedReader in = null;
        PrintWriter out = null;

        try {
            reader = new InputStreamReader(client.getInputStream());
            outputStream = new BufferedOutputStream(client.getOutputStream());
            
            in = new BufferedReader(reader);
            out = new PrintWriter(outputStream, true);
        } catch (IOException e) {
            System.out.println("Failed to create streams");
            System.exit(-1);
        }

        // Read the data from the socket into a string buffer
        boolean success = true;
        StringBuffer buf = new StringBuffer();
        try {
            int parsePhase = 0;
            int current = -1;
            boolean crlf = false;
            boolean cr = false;
            boolean weird=false;
            boolean completed = false;
            while (success && !completed) {
                current = in.read();
                char ch = (char) current;
                if (ch == '\r') {
                    cr = true;
                } else if ((ch == '\n') && cr) {
                    if (crlf) {
                        if (parsePhase == 0) {
                            success = parseHeader(buf.toString());
                            parsePhase = 1;
                        } else if (parsePhase == 1) {
                            body = buf.toString();
                            parsePhase = 2;
                        } else if (parsePhase == 2) {
                            body += "\r\n" + buf.toString();
                            parsePhase = 3;
                        } else if (parsePhase == 3) {
                            body += "\r\n" + buf.toString();
                            int index=0;
                            int index2=0;
                            int index3=0;
                            index=body.indexOf(boundary);
                            index2=body.indexOf(boundary,index+1);
                            index3=body.indexOf(boundary,index2+1);
                            if(index3!=-1){
                                success = parseBody();
                                completed = true;
                            }else{
                                weird=true;
                            }
                        }
                    
                        buf = new StringBuffer();
                        crlf = false;
                    } else {
                        buf.append("\n");
                        crlf = true;
                    }
                    cr = false;
                } else {
                    buf.append((char) current);
                    cr = false;
                    crlf = false;
                    
                    //Donal O'Kane April 2005 
                    //I put this in to fix some weirdness with parsing messages from an IPAQ ..
                    //was not reading in the payload part of a message and breaking .. so now checks to see that all 3
                    //boundarys have been read in and reads them in until it gets three.
                    if(weird && parsePhase==3){
                        //System.out.println("weird phase");
                        body += "" + buf.toString();
                        buf = new StringBuffer();
                        //System.out.println("body ::"+body+"::");
                        int index=0;
                        int index2=0;
                        int index3=0;
                        index=body.indexOf(boundary);
                        index2=body.indexOf(boundary,index+1);
                        index3=body.indexOf(boundary,index2+1);
                        if(index3!=-1){
                            //System.out.println("index not -1 :"+index +","+ index2+","+index3);
                            success = parseBody();
                            completed = true;
                            
                        }
                    } 
                }
            }
            
        } catch (IOException e) {
            System.out.println("Read failed: " +e);
        }
            
        if (success) {
            out.println("HTTP/1.1 200 OK");
            out.println("Content-Type: text/plain");
            out.println("Cache-Control: no-cache");
            out.println("Connection: close");
        } else {
            out.println("HTTP/1.1 400 OK");
            out.println("Content-Type: text/plain");
            out.println("Cache-Control: no-cache");
            out.println("Connection: close");
        }
        
        try {
            in.close();
            out.close();
            client.close();
        } catch (IOException e) {
            System.out.println("Read failed");
        }
    }

    /**
     * This method checks the string passed to if for compliance with the
     * FIPA ALPHAAgent Message Transport Protocol for HTTP Specification.
     * Specifically, this string should be the first line of text received
     * by the socket.
     *
     * @param command the first line received by the socket.
     * @return true if the first line is valid, false otherwise.
     */
    private boolean isValidRequest(String command) {
        boolean valid = false;
        String address, protocol;
        
        // The first line should take the following form:
        // POST http://url:port/acc HTTP/1.1
        // Tokenize command using a space as a delimiter, and
        // check each token for validity.
        StringTokenizer tok = new StringTokenizer(command, " ");
        
        //The first token must be equal to POST!
        String action = tok.nextToken();
        if (Constants.ACTION.equals(action)) {
            try {
                // The second token must end with the ACC_PATH constant, and
                // the third token must be HTTP/1.1
                address = tok.nextToken().trim();
                protocol = tok.nextToken().trim();
                if ((protocol.equals(Constants.HTTP11)) &&
                    (address.endsWith(Constants.ACC_PATH))) {
                    valid = true;
                }
            } catch (Exception e) {
                System.out.println("Failed to validate request: " + command);
            }
        }
        
        return valid;
    }
    
    /**
     * This method identifies the header of the unfoldedBuffer, and parses it
     * in compliance with the FIPA ALPHAAgent Message Transport Protocol for HTTP
     * specification.
     *
     * In addition, this method sets values such as host and boundary as
     * specified in the header, and identifies the body of the message.
     *
     * @param unfoldedBuffer a string containing the data received by the socket
     * @return true if the header is valid, false otherwise.
     */
    private boolean parseHeader(String header) {
        boolean valid = true;

        // Tokenize the header using a carriage return.  The first token
        // will be the first line which must be checked for validity. All
        // remaining tokens will be request parameters.
        StringTokenizer tok = new StringTokenizer(header, "\n");
        //System.out.println("*******************\nheader is "+header+"\n*******************");
        // Check if first line of request is valid
        if (isValidRequest(tok.nextToken())) {
            // The first line is valid, so check the request parameters
            // are also correct.
            String name = null, value = null;
            StringTokenizer lineTok = null, cTypeTok = null, boundaryTok = null;
            while (tok.hasMoreTokens() && valid) {
                // Tokenize each line by the colon (:), and check what
                // the name of the parameter is.  This loop should check
                // only mandatory parameters as defined in the FIPA Message
                // Transport Protocol for HTTP Specification.  The
                // specification currently requires the following
                // parameters to be set:
                //  - Cache-Control ( = no-cache )
                //  - Host
                //  - MIME-Version ( = 1.0 )
                //  - Content-Type ( = multipart/mixed; boundary="xxx" )
                lineTok = new StringTokenizer(tok.nextToken(), ":");
                name = lineTok.nextToken();
                
                if (name.equals("Cache-Control")) {
                    value = lineTok.nextToken().trim();
                    if (!value.equals("no-cache")) {
                        valid = false;
                    }
                //} else if (name.equals("Host")) {
                //    host = lineTok.nextToken().trim();
            
                } else if (name.equals("MIME-Version")) {
                    value = lineTok.nextToken().trim();
                    if (!value.equals("1.0")) {
                        valid = false;
                    }
                } else if (name.equals(Constants.CONTENT_TYPE)) {
                    // Need to split the content type from the boundary...
                    String t=lineTok.nextToken().trim();
                    cTypeTok = new StringTokenizer(t, ";");
                    if (!cTypeTok.nextToken().trim().equals("multipart/mixed")) {
                        valid = false;
                    } else {
                        // Finally deal with the boundary = "xxx" part
                        if(cTypeTok.hasMoreTokens()){
                            boundaryTok = new StringTokenizer(cTypeTok.nextToken().trim(), "=");
                        }else{
                            if(tok.hasMoreTokens()){
                                String l=tok.nextToken().trim();
                                boundaryTok = new StringTokenizer(l, "=");
                            }
                        }
                        if (!boundaryTok.nextToken().trim().equals("boundary")) {
                            valid = false;
                        } else {
                            value = boundaryTok.nextToken().trim();
                            boundary = value.substring(1, value.length() - 1);
                        }
                    }
                }
            }
        } else {
            // The first line was considered to be invalid
            valid = false;
        }

        return valid;
    }

    /**
     * We now start to process the body of the request.  We are expecting this
     * body to be organised into two parts: the envelope and the payload.  As
     * specified in RFC2046, each part of the body is delimited by the following
     * string: "\n--<BOUNDARY>" where <BOUNDARY> is the boundary value passed
     * as part of the Content-Type parameter.
     */
    private boolean parseBody() {
        boolean valid = true;
        String delimiter = "\n--" + boundary;
//        System.out.println("*******************");
//        System.out.println("Body :::"+body+":::");
//        System.out.println("*******************");
        // Look for the first occurrence of the message delimiter.  This marks
        // the start of the first part.
        int index = body.indexOf(delimiter);
        if (index != -1) {
            // Now get the second occurrence of the delimiter.  This marks the
            // end of the first part.
            
            int index2 = body.indexOf(delimiter, index+delimiter.length());
            if (index2 != -1) {
                // We have the first part!
                IEnvelope envelope = HTTPHelper.
                    parseHTTPEnvelope(body.substring(index+delimiter.length(),
                                                     index2).trim());
                // Now get the third occurrence of the delimiter.  This marks
                // the end of the second part!
                int index3 = body.indexOf(delimiter, index2+delimiter.length());
                if (index3 != -1) {
                    String payload = body.substring(index2+delimiter.length(), index3).trim();
                    int index4 = payload.indexOf(Constants.CRLF);
                    IMessage message = HTTPHelper.
                        parseHTTPPayload(envelope,
                                         payload.substring(index4 + 2).trim());

                    // TODO (Rem Collier): The following code caters only for
                    // cases where all the receiver agents are on the same
                    // agent platform.
                    //
                    // Future versions must address:
                    // * routing of messages to other agent platforms over diverse MTS
                    if (!service.receiveMessage(message)) {
                        System.out.println("Failed to deliver message:");
                        System.out.println(message);
                        valid = false;
                    }
                } else {
                    System.out.println("Failed to find the third boundary!");
                    valid = false;
                }
            } else {
                System.out.println("Failed to find second boundary!");
                valid = false;
            }
        } else {
            System.out.println("No content found!");
            valid = false;
        }
        
        return valid;
    }
}

