package com.agentfactory.teleoreactive.interpreter;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Properties;

import com.agentfactory.clf.interpreter.AbstractLanguageAgent;
import com.agentfactory.clf.interpreter.Action;
import com.agentfactory.clf.interpreter.CoreUtilities;
import com.agentfactory.clf.interpreter.Sensor;
import com.agentfactory.clf.lang.IFormula;
import com.agentfactory.clf.lang.IMPLIES;
import com.agentfactory.clf.lang.ITerm;
import com.agentfactory.clf.lang.ListTerm;
import com.agentfactory.clf.lang.Predicate;
import com.agentfactory.clf.reasoner.IQueryEngine;
import com.agentfactory.clf.reasoner.QueryResult;
import com.agentfactory.clf.reasoner.ResolutionBasedQueryEngine;
import com.agentfactory.platform.core.AgentID;
import com.agentfactory.platform.core.IMessage;
import com.agentfactory.platform.impl.StringMessage;
import com.agentfactory.platform.util.FIPAHelper;
import com.agentfactory.teleoreactive.interpreter.mentalstate.ModelTower;
import com.agentfactory.teleoreactive.interpreter.mentalstate.RuleBase;
import com.agentfactory.teleoreactive.interpreter.mentalstate.TRProgram;

public abstract class AbstractTeleoReactiveAgent extends AbstractLanguageAgent {
	protected ModelTower beliefs;
	protected IQueryEngine engine;
	protected RuleBase BCrules;
	protected Map<String, TRProgram> programs;
	protected String activeProgram;
	protected List<String> programStack;
	protected List<String> actionTrace;

	public AbstractTeleoReactiveAgent(String name) {
		super(name);
		actionTrace = new LinkedList<String>();
		beliefs = new ModelTower();
		engine = new ResolutionBasedQueryEngine();
		BCrules = new RuleBase();
		engine.addSource(beliefs);
		engine.addSource(BCrules);
		activeProgram = "main:0";
		setPrograms(new HashMap<String, TRProgram>());
		programStack = new LinkedList<String>();
		actionTrace = new LinkedList<String>();
		installBasicAPI();
		addAction(".doAll(?list)", new Action() {
			public boolean execute(Predicate activity) {
				for(String t :termAsList(activity, 0)){
//					System.out.println(t);
					performActivity((Predicate)CoreUtilities.factory.createFormula(t));
				}
				return true;
			}
		});
		
		addAction(".adopt(?list)",new Action(){

			public boolean execute(Predicate activity) {
				for(String t :termAsList(activity, 0)){
//					System.out.println(t);
					beliefs.addNewInformedBelief((Predicate)CoreUtilities.factory.createFormula(t));
				}
				return true;
			}});
		addAction(".drop(?list)",new Action(){

			public boolean execute(Predicate activity) {
				for(String t :termAsList(activity, 0)){
//					System.out.println(t);
					beliefs.removeInformedBelief((Predicate)CoreUtilities.factory.createFormula(t));
				}
				return true;
			}});
		
		addAction(".abolishAll",new Action(){

			public boolean execute(Predicate activity) {
				beliefs.clearInformedBeliefs();
				return true;
			}});
		
		addAction(".send(?perf, ?rec, ?content)", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				String performative = termAsString(activity, 0);
				AgentID receiver;
				receiver = CoreUtilities.toAgentID(activity.termAt(1));
				String content = termAsString(activity, 2);

				StringMessage message = StringMessage.newInstance();
				message.setPerformative(performative.toUpperCase());
				message.setSender(agent.getAgentID());
				message.getReceivers().add(receiver);
				message.setContent(content.toString());
				message.setLanguage("AFAPL");
				message.setOntology("none");
				message.setProtocol("none");
				addSentBelief("\"" + receiver.getName() + "\"", content, performative);
				return agent.send(message);
			}
		});

		addSensor(".inbox", new Sensor() {
			@Override
			public void perceive() {
				for (IMessage message : agent.getInbox()) {
					addMessageBelief(message.getPerformative().toLowerCase(), message.getSender(), message.getContent());
					addBelief("message(" + message.getPerformative().toLowerCase() + "," + FIPAHelper.toFOSString(message.getSender()) + ","
							+ message.getContent() + ")");
				}
			}
		});

	}

	@Override
	// public void execute() {
	// this.retrieveNewMessages();
	// senseEnvironment();
	// beliefs.endOfPerception();
	// this.getOutbox().clear();
	//
	// IFormula activity = getPrograms().get(activeProgram).execute(engine, new LinkedList<ITerm>());
	// while (getPrograms().containsKey(CoreUtilities.createSignature((Predicate) activity))) {
	// programStack.add(CoreUtilities.presenter.toString(activity));
	// if (Predicate.class.isInstance(activity)) {
	// Predicate potentialAction = ((Predicate) activity);
	// LinkedList<ITerm> args = new LinkedList<ITerm>();
	// for (int i = 0; i < potentialAction.size(); i++) {
	// args.add(i, potentialAction.termAt(i));
	// }
	// activity = getPrograms().get(CoreUtilities.createSignature((Predicate) activity)).execute(engine, args);
	// }
	// }
	//
	// if(actionTrace.isEmpty() || (!actionTrace.get(actionTrace.size()-1).equals(CoreUtilities.presenter.toString(activity)) &&
	// !CoreUtilities.presenter.toString(activity).equals(".nil")) )
	// actionTrace.add(actionTrace.size(), CoreUtilities.presenter.toString(activity));
	// performActivity((Predicate) activity);
	// endOfIteration();
	// }
	public void execute() {
		this.retrieveNewMessages();
		senseEnvironment();
		beliefs.endOfPerception();
		this.getOutbox().clear();

		List<IFormula> activity = getPrograms().get(activeProgram).executeAll(engine, new LinkedList<ITerm>());
		while (getPrograms().containsKey(CoreUtilities.createSignature((Predicate) activity.get(0)))) {
			programStack.add(CoreUtilities.presenter.toString(activity.get(0)));
			if (Predicate.class.isInstance(activity.get(0))) {
				Predicate potentialAction = ((Predicate) activity.get(0));
				LinkedList<ITerm> args = new LinkedList<ITerm>();
				for (int i = 0; i < potentialAction.size(); i++) {
					args.add(i, potentialAction.termAt(i));
				}
				activity = getPrograms().get(CoreUtilities.createSignature((Predicate) activity.get(0))).executeAll(engine, args);
			}
		}
		if (CoreUtilities.presenter.toString(activity.get(0)).contains("executeAll(")) {
			for (IFormula iFormula : activity) {
				if (actionTrace.isEmpty()
						|| (!actionTrace.get(actionTrace.size() - 1).equals(CoreUtilities.presenter.toString(iFormula)) && !CoreUtilities.presenter.toString(
								iFormula).equals(".nil")))
					actionTrace.add(actionTrace.size(), getIterations() + ": " + CoreUtilities.presenter.toString(((Predicate)iFormula).termAt(0)));

			}
		} else {
			if (actionTrace.isEmpty()
					|| (!actionTrace.get(actionTrace.size() - 1).equals(CoreUtilities.presenter.toString(activity.get(0))) && !CoreUtilities.presenter.toString(
							activity.get(0)).equals(".nil")))
				actionTrace.add(actionTrace.size(), getIterations() + ": " + CoreUtilities.presenter.toString(activity.get(0)));
		}
		performActivity(activity);
		endOfIteration();
	}

	private void performActivity(List<IFormula> activity) {
		if (CoreUtilities.presenter.toString(activity.get(0)).contains("executeAll(")) {
			for (IFormula iFormula : activity) {
				IFormula act = CoreUtilities.factory.convertToPredicate(((Predicate)iFormula).termAt(0));
				performActivity((Predicate) act);
			}
		} else {
			performActivity((Predicate) activity.get(0));
		}

	}

	public String getType() {
		return "AFTeleoReactive";
	}

	public void initialise(String data) {
		IFormula act = CoreUtilities.factory.createFormula(data);
		if (Predicate.class.isInstance(act))
			beliefs.addNewInformedBelief((Predicate) act);
	}

	public void update(Observable o, Object arg) {
	}

	public List<String> getStack() {
		List<String> s = new LinkedList<String>();
		for (String string : programStack) {
			s.add(string);
		}
		programStack.clear();
		return s;
	}

	public List<String> getBeliefs() {
		return beliefs.getBeliefs();
	}

	public List<String> getActionTrace() {
		List<String> n = new LinkedList<String>();
		for (String s : actionTrace) {
			n.add(s);
		}
		return n;
	}

	public void addBelief(Predicate bel) {
		beliefs.addBelief(bel);
	}

	public void addBelief(String bel) {
		beliefs.addBelief((Predicate) CoreUtilities.factory.createFormula(bel));
	}

	public void addMessageBelief(String perf, AgentID sender, String content) {
		// beliefs.addMesageBelief((Predicate) CoreUtilities.factory.createFormula("comunicatingWith("+sender+")"));
		// System.out.println("Adding Belief: "+content+" to agent: "+name);
		if (perf.equals("inform"))
			beliefs.addNewInformedBelief((Predicate) CoreUtilities.factory.createFormula(content));
		else if (perf.equals("request")) {
			beliefs.addNewRequestBelief(sender, content);
		}
	}

	public void addSentBelief(String agentID, String content, String perf) {
		if (perf.equals("inform"))
			beliefs.addMesageBelief((Predicate) CoreUtilities.factory.createFormula("informed(" + agentID + "," + content + ")"));
		else
			beliefs.addMesageBelief((Predicate) CoreUtilities.factory.createFormula("request(" + agentID + "," + content + ")"));

	}

	void processArgs(Properties arguments) {
		String argument = arguments.getProperty("TIMESLICE");
		if (argument != null) {
			setTimeSlice(Integer.parseInt(argument));
		}
	}

	public boolean hasBelief(IFormula createFormula) {
		QueryResult result = engine.query(createFormula);
		return result.found;
	}

	public void setPrograms(Map<String, TRProgram> programs) {
		this.programs = programs;
	}

	public Map<String, TRProgram> getPrograms() {
		return programs;
	}

	public List<IMPLIES> getInferences() {
		return beliefs.getRules();
	}

	public List<IMPLIES> getResolutions() {
		return BCrules.getRules();
	}

	public List<IMPLIES> getTruthMan() {
		return beliefs.getTruthRules();
	}

}
