package com.agentfactory.teleoreactive.interpreter.mentalstate;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.clf.interpreter.CoreUtilities;
import com.agentfactory.clf.lang.IFormula;
import com.agentfactory.clf.lang.ITerm;
import com.agentfactory.clf.lang.Predicate;
import com.agentfactory.clf.lang.Variable;
import com.agentfactory.clf.reasoner.Bindings;
import com.agentfactory.clf.reasoner.IQueryEngine;
import com.agentfactory.clf.reasoner.QueryResult;

public class TRProgram {
	private List<TRRule> rules;
	private String name;
	private List<ITerm> arguments;
	private boolean abstractbool = false;
	private boolean override = false;
	private boolean implementsbool = false;

	public TRProgram(String name, List<TRRule> rules, List<ITerm> args) {
		this.name = name;
		this.rules = rules;
		this.arguments = args;

	}

	public TRProgram(List<String> rules, List<String> args,String name) {
		this.name = name;
		for (String string : rules) {
			this.rules.add(new TRRule(string));
		}
		for (String string : args) {
			this.arguments.add(CoreUtilities.factory.createTerm(string));
		}
	}

	public void setOverride(boolean override) {
		this.override = override;
	}

	public boolean isOverride() {
		return override;
	}

	public void setAbstract(boolean abstractbool) {
		this.abstractbool = abstractbool;
	}

	public boolean isAbstract() {
		return abstractbool;
	}
	
	public void setImplements(boolean bool){
		this.implementsbool = bool;
	}
	
	public boolean isImplements(){
		return implementsbool ;
	}

	/**
	 * @return the list of teleo-reactive rules
	 */
	public List<TRRule> getRules() {
		return rules;
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @return the arguments
	 */
	public List<ITerm> getArguments() {
		return arguments;
	}
	
	/**
	 * Get the rules of the function as a list of strings
	 * @return
	 */
	public List<String> getRuleStrings(){
		LinkedList<String> list = new LinkedList<String>();
		for (TRRule rule : rules) {
			list.add(rule.toString());
		}
		return list;
	}
	
	public List<String> getArgStrings(){
		List<String> list = new LinkedList<String>();
		for (ITerm arg : arguments) {
			list.add(CoreUtilities.presenter.toString(arg));
		}
		return list;
	}

	/**
	 * 
	 * @param queryEngine
	 * @return
	 */
	public IFormula execute(IQueryEngine queryEngine, LinkedList<ITerm> linkedList) {
		Bindings solution = new Bindings();
		for (int i = 0; i < arguments.size(); i++) {
			solution.addBinding((Variable) arguments.get(i), linkedList.get(i));
		}
		LinkedList<TRRule> boundRules = new LinkedList<TRRule>();
		for (TRRule r : rules) {
			TRRule br = r.apply(solution);
			boundRules.add(br);
		}
		for (TRRule rule : boundRules) {
			QueryResult result = queryEngine.query(rule.getCondition());
			if (result.found) {
				if (!result.bindings.isEmpty()){
					return rule.apply(result.bindings.get(0)).getAction(); 
				} else
					return rule.getAction();
			}
		}
		return CoreUtilities.factory.createFormula(".nil");
	}
	
	public List<IFormula> executeAll(IQueryEngine queryEngine, LinkedList<ITerm> linkedList) {
		List<IFormula> list = new LinkedList<IFormula>();
		Bindings solution = new Bindings();
		for (int i = 0; i < arguments.size(); i++) {
			solution.addBinding((Variable) arguments.get(i), linkedList.get(i));
		}
		LinkedList<TRRule> boundRules = new LinkedList<TRRule>();
		for (TRRule r : rules) {
			TRRule br = r.apply(solution);
			boundRules.add(br);
		}
		for (TRRule rule : boundRules) {
//			System.out.println("Querying condition "+ CoreUtilities.presenter.toString(rule.getCondition()));
			QueryResult result = queryEngine.queryAll(rule.getCondition());
			if (result.found) {
				if (!result.bindings.isEmpty()){
					for (Bindings b : result.bindings) {
						
//						System.out.println("Number of Bindings: "+ result.bindings.size()+" "+CoreUtilities.presenter.toString(b));
						list.add(rule.apply(b).getAction());
					}
					return list; 
				} else{
					list.add(rule.getAction());
					return list;
				}
			}
		}
		list.add(CoreUtilities.factory.createFormula(".nil"));
		return list;
	}
	

	public List<String> getFunction() {
		List<String> functions = new LinkedList<String>();
		functions.add(name.substring(0, name.length()-2)+"{");
		for(TRRule i:rules){
			functions.add("    "+i.toString());
		}
		functions.add("}");
		return functions;
	}

}
